#
#	THIS FILE IS PART OF THE JOKOSHER PROJECT AND LICENSED UNDER THE GPL. SEE
#	THE 'COPYING' FILE FOR DETAILS
#
#	EventViewer.py
#	
#	This module is the gui class that represents an event.
#	It handles the drawing of the waveform, audio fades, and
#	anything else that happens when you click a rectangular 
#	event on the gui.
#
#-------------------------------------------------------------------------------

import gtk
import cairo
from Project import Project
import Utils
import os, sys
import gettext
_ = gettext.gettext
import Globals
import itertools

#=========================================================================

class EventViewer(gtk.DrawingArea):
	"""
	The EventViewer class handles displaying a single event as part
	of an EventLaneViewer object.
	"""
	
	""" GTK widget name """
	__gtype_name__ = 'EventViewer'
	
	#the width of the stroke above the fill (the line on the top of the waveform)
	_LINE_WIDTH = 2
	
	#the minimum distance allowed between each sample point in the waveform
	#making this bigger will make the waveform less crowed but also less detailed
	_MIN_POINT_SEPARATION = 2
	
	#the width and height of the volume curve handles
	_PIXX_FADEMARKER_WIDTH = 30
	_PIXY_FADEMARKER_HEIGHT = 11
	
	"""
	Various color configurations:
	   ORGBA = Offset, Red, Green, Blue, Alpha
	   RGBA = Red, Green, Blue, Alpha
	   RGB = Red, Green, Blue
	"""
	_OPAQUE_GRADIENT_STOP_ORGBA = (0.2, 114./255, 159./255, 207./255, 1)
	_TRANSPARENT_GRADIENT_STOP_ORGBA = (1, 52./255, 101./255, 164./255, 1)
	_BORDER_RGB = (32./255, 74./255, 135./255) 
	_BORDER_HIGHLIGHT_RGB = (0, 0, 1)
	_BACKGROUND_RGB = (1, 1, 1)
	_TEXT_RGB = (0, 0, 0)
	_SELECTED_RGBA = (0, 0, 1, 0.2)
	_SELECTION_RGBA = (0, 0, 1, 0.5)
	_FADEMARKERS_RGBA = (1, 0, 0, 0.8)
	_PLAY_POSITION_RGB = (1, 0, 0)
	_HIGHLIGHT_POSITION_RGB = (0, 0, 1)
	_FADELINE_RGB = (1, 0.6, 0.6)
	
	#_____________________________________________________________________

	def __init__(self, lane, project, event, height, mainview,  small=False):
		"""
		Creates a new instance of EventViewer.
		
		Parameters:
			lane -- parent EventLaneViewer for this instance.
			project -- the currently active Project.
			event -- Event drawn by this EventViewer.
			height -- height in pixels for this EventViewer.
			mainview -- the parent MainApp Jokosher window.
			small - set to True if we want small edit views (i.e. for mixing view).
		"""
		self.small = small
		
		self.selectiontip = gtk.Tooltips()
		
		gtk.DrawingArea.__init__(self)

		self.set_events(gtk.gdk.POINTER_MOTION_MASK |
		                gtk.gdk.BUTTON_RELEASE_MASK |
		                gtk.gdk.BUTTON_PRESS_MASK |
		                gtk.gdk.LEAVE_NOTIFY_MASK |
		                gtk.gdk.KEY_PRESS_MASK)
		
		self.connect("expose_event",self.OnDraw)
		self.connect("motion_notify_event", self.OnMouseMove)
		self.connect("leave_notify_event", self.OnMouseLeave)
		self.connect("button_press_event", self.OnMouseDown)
		self.connect("focus-in-event", self.OnFocus)
		self.connect("focus-out-event", self.OnFocusLost)
		self.connect("key_press_event", self.OnKeyPress)
		self.connect("key_release_event", self.OnKeyRelease)
		self.connect("button_release_event", self.OnMouseUp)
		
		self.height = height			# Height of this object in pixels
		self.event = event				# The event this widget is representing
		self.project = project			# A reference to the open project
		self.isDragging = False			# True if this event is currently being dragged
		# Selections--marking part of the waveform. Don't confuse this with
		# self.event.isSelected, which means the whole waveform is selected.
		self.isSelecting = False		# True if a selection is currently being set
		self.isDraggingFade = False		# True if the user is dragging a fade marker
		self.lane = lane				# The parent lane for this object
		self.currentScale = 0			# Tracks if the project viewScale has changed
		self.redrawWaveform = False		# Force redraw the cached waveform on next expose event
		#boolean; if the drawer should be at the left of current selection
		#otherwise it will be put on the right
		self.drawerAlignToLeft = True		
		
		self.fadeMarkers = [100,100]		#the values of the right and left fade markers on the selection

		# Set accessibility helpers
		self.SetAccessibleName()
		self.set_property("can-focus", True)
		
		# sourceSmall/Large are offscreen canvases to hold our waveform images
		self.sourceSmall = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
		self.sourceLarge = cairo.ImageSurface(cairo.FORMAT_ARGB32, 0, 0)
		
		# rectangle of cached draw areas
		self.cachedDrawAreaSmall = gtk.gdk.Rectangle(0, 0, 0, 0)
		self.cachedDrawAreaLarge = gtk.gdk.Rectangle(0, 0, 0, 0)

		# Monitor the things this object cares about
		self.project.connect("zoom", self.OnProjectZoom)
		self.event.connect("waveform", self.OnEventWaveform)
		self.event.connect("position", self.OnEventPosition)
		self.event.connect("length", self.OnEventLength)
		self.event.connect("corrupt", self.OnEventCorrupt)
		self.event.connect("loading", self.OnEventLoading)
		self.event.connect("selected", self.OnEventSelected)

		# This defines where the blue cursor indicator should be drawn (in pixels)
		self.highlightCursor = None
		self.fadeMarkersContext = None
		
		self.splitImg = gtk.gdk.pixbuf_new_from_file(os.path.join(Globals.IMAGE_PATH, "icon_split.png"))
		self.cancelImg = cairo.ImageSurface.create_from_png(os.path.join(Globals.IMAGE_PATH, "icon_cancel.png"))
		self.cancelButtonArea = gtk.gdk.Rectangle(85, 3, self.cancelImg.get_width(), self.cancelImg.get_height())
		
		# drawer: this will probably be its own object in time
		self.drawer = gtk.HBox()
		trimButton = gtk.Button()
		trimimg = gtk.Image()
		trimimg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_trim.png"))
		trimButton.set_image(trimimg)
		self.selectiontip.set_tip(trimButton,_("Trim"),None)

		self.drawer.add(trimButton)
		trimButton.connect("clicked", self.TrimToSelection)
		
		delFPButton = gtk.Button()
		delimg = gtk.Image()
		delimg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_fpdelete.png"))
		delFPButton.set_image(delimg)
		self.drawer.add(delFPButton)
		delFPButton.connect("clicked", self.DeleteSelectedFadePoints)
		self.selectiontip.set_tip(delFPButton,_("Delete Fade Points"),None)
		
		snapFPButton = gtk.Button()
		snapimg = gtk.Image()
		snapimg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_fpsnap.png"))
		snapFPButton.set_image(snapimg)
		self.drawer.add(snapFPButton)
		snapFPButton.connect("clicked", self.SnapSelectionToFadePoints)
		self.selectiontip.set_tip(snapFPButton,_("Snap To Fade Points"),None)
		
		self.drawer.set_sensitive(not self.event.isLoading)
		self.drawer.show()
		
		self.mainview = mainview
		self.messageID = None
		self.volmessageID = None
		self.selmessageID = None
		
	#_____________________________________________________________________

	def OnDraw(self, widget, event):
		"""
		Blits the waveform data onto the screen, and then draws the play
		cursor over it.
		
		widget -- GTK widget to be drawn.
		event -- GTK event associated to the widget.
		
		Returns:
			False -- stop propagating the GTK signal. *CHECK*
		"""
		if self.small:
			cache = self.cachedDrawAreaSmall
			source = self.sourceSmall
		else:
			cache = self.cachedDrawAreaLarge
			source = self.sourceLarge
		area = event.area
		
		#check if the expose area is within the already cached rectangle
		if area.x < cache.x or (area.x + area.width > cache.x + cache.width) or self.redrawWaveform:
			self.DrawWaveform(event.area)
			if self.small:
				cache = self.cachedDrawAreaSmall
				source = self.sourceSmall
			else:
				cache = self.cachedDrawAreaLarge
				source = self.sourceLarge
		
		# Get a cairo surface for this drawing op
		context = widget.window.cairo_create()

		# Give it our waveform image as a source
		context.set_source_surface(source, cache.x, cache.y)	

		# Blit our waveform across
		context.paint()

		# Overlay an extra rect if we're selected
		if self.event.isSelected:
			context.rectangle(event.area.x, event.area.y,
							  event.area.width, event.area.height)
			context.set_source_rgba(*self._SELECTED_RGBA)
			context.fill()
			

		bx, by, bwidth, bheight = self.get_allocation()
		context.rectangle(0, 0, bwidth, bheight)
		# Draw the border
		if self.is_focus():
			# Highlight the border if we have focus
			context.set_source_rgb(*self._BORDER_HIGHLIGHT_RGB)
		else:
			context.set_source_rgb(*self._BORDER_RGB)
		context.stroke()
		context.set_line_width(2)
		
		#Draw play position
		# TODO: don't calculate pixel position based on self.event.start, it will have rounding errros
		# instead determine the pixel position of the start of our widget and subtract that from GetPixelPosition().
		x = self.project.transport.GetPixelPosition(self.event.start)
		context.set_line_width(1)
		context.set_antialias(cairo.ANTIALIAS_NONE)
		context.move_to(x+0.5, 0)
		context.line_to(x+0.5, self.allocation.height)
		context.set_source_rgb(*self._PLAY_POSITION_RGB)
		context.stroke()
		
		#Don't draw any cut markers, cause we cant cut while recording!
		if self.event.instrument.project.GetIsRecording():
			return
		
		# Draw the highlight cursor if it's over us and we're not dragging a fadeMarker
		if self.highlightCursor and not self.isDraggingFade and not self.event.isLoading:
			context.move_to(self.highlightCursor, 0)
			context.line_to(self.highlightCursor, self.allocation.height)
			context.set_source_rgb(*self._HIGHLIGHT_POSITION_RGB)
			context.set_dash([3,1],1)
			context.stroke()
			widget.window.draw_pixbuf(None, self.splitImg, 0, 0, int(self.highlightCursor) - int(self.splitImg.get_width() / 2) , 0)
			
		# Overlay an extra rect if there is a selection
		self.fadeMarkersContext = None
		if self.event.selection != [0,0]:
			x1,x2 = self.GetSelectionAsPixels()
			if x2 < x1:
				x2,x1 = x1,x2
			context.rectangle(x1, 0, x2 - x1, event.area.height)
			context.set_source_rgba(*self._SELECTION_RGBA)
			context.fill()
			
			#subtract fade marker height so that it is not drawn partially offscreen
			padded_height = self.allocation.height - self._PIXY_FADEMARKER_HEIGHT
			
			# and overlay the fademarkers
			context.set_source_rgba(*self._FADEMARKERS_RGBA)
			
			pixxFM_left = x1 + 1
			
			#if there is enough room on the left of the selection,
			#place the fademarker outside the selection bounds.
			if x1 + 1 >= self._PIXX_FADEMARKER_WIDTH:
				pixxFM_left -= self._PIXX_FADEMARKER_WIDTH
			pixyFM_left = int(padded_height * (100-self.fadeMarkers[0]) / 100.0)
			context.rectangle(pixxFM_left, pixyFM_left,
			                  self._PIXX_FADEMARKER_WIDTH , self._PIXY_FADEMARKER_HEIGHT)
			
			pixxFM_right = x2
			
			#if there is enough room on the right of the selection,
			#place the fademarker outside the selection bounds.
			if x2 + self._PIXX_FADEMARKER_WIDTH > event.area.width:
				pixxFM_right -= self._PIXX_FADEMARKER_WIDTH
			pixyFM_right = int(padded_height * (100-self.fadeMarkers[1]) / 100.0)
			context.rectangle(pixxFM_right, pixyFM_right,
			                  self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
			
			context.fill()
			
			context.set_source_rgba(1,1,1,1)
			context.move_to(pixxFM_left + 1, pixyFM_left + self._PIXY_FADEMARKER_HEIGHT - 1)
			context.show_text("%s%%" % int(self.fadeMarkers[0]))
			context.move_to(pixxFM_right + 1, pixyFM_right + self._PIXY_FADEMARKER_HEIGHT - 1)
			context.show_text("%s%%"% int(self.fadeMarkers[1]))
			context.stroke()
			
			# redo the rectangles so they're the path and we can in_fill() check later
			context.rectangle(pixxFM_left, pixyFM_left,
			                  self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
			context.rectangle(pixxFM_right, pixyFM_right,
					          self._PIXX_FADEMARKER_WIDTH, self._PIXY_FADEMARKER_HEIGHT)
			self.fadeMarkersContext = context

		return False
		
	#_____________________________________________________________________

	def DrawWaveform(self, exposeArea):
		"""
		Uses Cairo to draw the waveform level information onto a canvas in memory.
		
		Parameters:
			exposeArea -- Cairo exposed area in which to draw the waveform.
		"""
		allocArea = self.get_allocation()
		
		rect = gtk.gdk.Rectangle(exposeArea.x - exposeArea.width, exposeArea.y,
		                         exposeArea.width*3, exposeArea.height)
		
		#Check if our area to cache is outside the allocated area
		if rect.x < 0:
			rect.x = 0
		if rect.x + rect.width > allocArea.width:
			rect.width = allocArea.width - rect.x
			
		#set area to record where the cached surface goes
		if self.small:
			self.cachedDrawAreaSmall = rect
			self.sourceSmall = cairo.ImageSurface(cairo.FORMAT_ARGB32, 
			                                      rect.width, rect.height)
			context = cairo.Context(self.sourceSmall)
		else:
			self.cachedDrawAreaLarge = rect
			self.sourceLarge = cairo.ImageSurface(cairo.FORMAT_ARGB32, 
			                                      rect.width, rect.height)
			context = cairo.Context(self.sourceLarge)
		
		context.set_line_width(2)
		context.set_antialias(cairo.ANTIALIAS_SUBPIXEL)

		# Draw white background
		context.rectangle(0, 0, rect.width, rect.height)
		context.set_source_rgb(*self._BACKGROUND_RGB)
		context.fill()
		
		if self.event.levels_list and (self.event.duration or self.event.loadingLength):
			if self.event.loadingLength:
				duration = self.event.loadingLength
			else:
				duration = self.event.duration
			
			context.move_to(0,rect.height)
			
			levels = self.event.GetFadeLevels()
			length = len(levels)
			
			# time offset of the start of the drawing area in milliseconds
			starting_time = int(rect.x / self.project.viewScale * 1000)
			starting_index = levels.find_endtime_index(starting_time)

			x = 0
			last_x = -2
			skip_list = []
			iterator = itertools.islice(levels, starting_index, length)
			for endtime, peak in iterator:
				x = int((endtime - starting_time) * self.project.viewScale / 1000)
				
				peakOnScreen = int(peak * rect.height / sys.maxint)
				skip_list.append(peakOnScreen)
				if (x - last_x) < self._MIN_POINT_SEPARATION:
					continue
				
				peakOnScreen = sum(skip_list) / len(skip_list)
				context.line_to(x, rect.height - peakOnScreen)
				
				skip_list = []
				last_x = x
				if x > rect.width:
					break
			
			context.line_to(x, rect.height)
			
			#levels gradient fill
			gradient = cairo.LinearGradient(0.0, 0.0, 0, rect.height)
			gradient.add_color_stop_rgba(*self._OPAQUE_GRADIENT_STOP_ORGBA)
			gradient.add_color_stop_rgba(*self._TRANSPARENT_GRADIENT_STOP_ORGBA)
			context.set_source(gradient)
			context.fill_preserve()
			
			#levels path (on top of the fill)
			context.set_source_rgb(*self._BORDER_RGB)
			context.set_line_join(cairo.LINE_JOIN_ROUND)
			context.set_line_width(self._LINE_WIDTH)
			context.stroke()
		
		if self.event.audioFadePoints:
			pixelPoints = []
			# draw the fade line
			context.set_source_rgb(*self._FADELINE_RGB)
			
			firstPoint = self.event.audioFadePoints[0]
			pixx = self.PixXFromSec(firstPoint[0]) - rect.x
			pixy = self.PixYFromVol(firstPoint[1])
			context.move_to(pixx, pixy)
			for sec, vol in self.event.audioFadePoints[1:]:
				pixx = self.PixXFromSec(sec) - rect.x
				pixy = self.PixYFromVol(vol)
				pixelPoints.append((pixx, pixy))
				context.line_to(pixx,pixy)		
			context.stroke()
			
			#draw the fade points
			for pixx, pixy in pixelPoints:
				context.arc(pixx, pixy, 3.5, 0, 7)
				context.fill()
		
		# Reset the drawing scale
		context.identity_matrix()
		context.scale(1.0, 1.0)
		
		#check if we are at the beginning
		if rect.x == 0:
			context.set_source_rgb(*self._TEXT_RGB)
			context.move_to(5, 15)
			
			if self.event.isLoading:
				# Write "Loading..." or "Downloading..."
				if self.event.duration <= 0:
					# for some file types gstreamer doesn't give us a duration
					# so don't display the percentage
					if self.event.isDownloading:
						message = _("Downloading...")
					else:
						message = _("Loading...")
				else:
					displayLength = int(100 * self.event.loadingLength / self.event.duration)
					if self.event.isDownloading:
						message = _("Downloading (%d%%)...") % displayLength
					else:
						message = _("Loading (%d%%)...") % displayLength
				
				# show the appropriate message
				context.show_text(message)
				
				# display a cancel button
				self.cancelButtonArea.x = context.get_current_point()[0]+3	# take the current context.x and pad it a bit
				context.set_source_surface(self.cancelImg, self.cancelButtonArea.x, self.cancelButtonArea.y)
				context.paint()
								
			elif self.event.isRecording:
				context.show_text(_("Recording..."))
			else:
				#Draw event name
				context.show_text(self.event.name)
		
		self.redrawWaveform = False

	#_____________________________________________________________________
	
	def Destroy(self):
		"""
		Called when the EventViewer gets destroyed.
		It also destroys any child widget and disconnects itself from any
		gobject signals.
		"""
		self.project.disconnect_by_func(self.OnProjectZoom)
		self.event.disconnect_by_func(self.OnEventSelected)
		self.event.disconnect_by_func(self.OnEventCorrupt)
		self.event.disconnect_by_func(self.OnEventLength)
		self.event.disconnect_by_func(self.OnEventLoading)
		self.event.disconnect_by_func(self.OnEventPosition)
		self.event.disconnect_by_func(self.OnEventWaveform)
		
		#delete the cached images
		del self.sourceSmall
		del self.sourceLarge
		del self.cancelImg
		self.destroy()
	
	#_____________________________________________________________________

	def OnMouseMove(self, widget, mouse):
		"""
		Display a message in the StatusBar when the mouse hovers over the
		EventViewer.
		Also displays cursors depending on the current action being performed.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- GTK mouse event that fired this method call.
			
		Returns:
			True -- stop GTK signal propagation.
		"""
		if not self.window or self.event.instrument.project.GetIsRecording():
			return True #don't let the intrument viewer handle it
		# display status bar message if has not already been displayed
		if not self.messageID: 
			self.messageID = self.mainview.SetStatusBar(_("To <b>Split, Double-Click</b> the wave - To <b>Select, Shift-Click</b> and drag the mouse"))
		
		if self.isDraggingFade:
			#subtract half the fademarker height so it doesnt go half off the screen
			cur_pos = (mouse.y - (self._PIXY_FADEMARKER_HEIGHT / 2))
			height = self.allocation.height - self._PIXY_FADEMARKER_HEIGHT
			percent = cur_pos / float(height)
			#set percent between 0 and 1
			percent = min(1, max(0, percent))
			
			self.fadeMarkers[self.fadeBeingDragged] = 100 - int(percent * 100)
			self.queue_draw()
			
			if not self.volmessageID:
				self.volmessageID = self.mainview.SetStatusBar(_("<b>Drag</b> the red sliders to modify the volume fade."))
			
			return True

		if self.fadeMarkersContext and self.fadeMarkersContext.in_fill(mouse.x, mouse.y):
			# quit this function now, so the highlightCursor doesn't move
			# while you're over a fadeMarker
			return True
			
		if self.isDragging:
			ptr = gtk.gdk.display_get_default().get_pointer()
			x = ptr[1]
			y = ptr[2]
			dx = float(x - self.mouseAnchor[0]) / self.project.viewScale
			time = self.event.start + dx
			time = max(0, time)
			
			if self.event.MayPlace(time):
				self.event.start = time
				self.lane.UpdatePosition(self)
				self.mouseAnchor = [x, y]
			else:
				temp = self.event.start
				self.event.MoveButDoNotOverlap(time)
				self.lane.UpdatePosition(self)
				
				#MoveButDoNotOverlap() moves the event out of sync with the mouse
				#and the mouseAnchor must be updated manually.
				delta = (self.event.start - temp) * self.project.viewScale
				self.mouseAnchor[0] += int(delta)
				
				
			self.highlightCursor = None
		elif self.isSelecting:
			x2 = max(0,min(self.allocation.width,mouse.x))
			self.event.selection[1] = self.SecFromPixX(x2)
			self.UpdateFadeMarkers()
			
			selection_direction = "ltor"
			selection = self.event.selection
			if selection[0] > selection[1]:
				selection_direction = "rtol"
				self.fadeMarkers.reverse()
			
			#set the drawer align position
			self.drawerAlignToLeft = (selection_direction == "rtol")
			#move the drawer to its proper position
			self.UpdateDrawerPosition(selection_direction == "rtol")
			
		else:
			self.highlightCursor = mouse.x
		
		self.queue_draw()
		return True

	#_____________________________________________________________________
	
	def OnMouseDown(self, widget, mouse):
		"""
		Called when the user pressed a mouse button.
		Possible click combinations to capture:
			{L|R}MB: deselect all Events, remove any existing selection in
					this Event then select this Event and begin moving the Event.
			LMB+shift: remove any existing selection in this Event and begin
					selecting part of this Event.
			{L|R}MB+ctrl: select this Event without deselecting other Events.
			RMB: display a context menu.
			LMB double-click: split this Event here.
			LMB over a fadeMarker: drag the correspondent marker.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- GTK mouse event that fired this method call.
			
		Returns:
			True -- continue GTK signal propagation. *CHECK*
		"""		
		#Don't allow moving, etc while recording!
		if self.event.instrument.project.GetIsRecording():
			return True #don't let the instrument viewer handle this click

		self.grab_focus()
		
		# {L|R}MB: deselect all events, select this event, begin moving the event
		# {L|R}MB+ctrl: select this event without deselecting other events
		if 'GDK_CONTROL_MASK' not in mouse.state.value_names:
			self.project.ClearEventSelections()
			self.project.SelectInstrument(None)
		self.event.SetSelected(True)
		
		#Don't allow editing while playing back.
		#It must be here to avoid afecting the selection behavior
		if self.mainview.isPlaying or self.mainview.isPaused:
			return True
		
		# RMB: context menu
		if mouse.button == 3:
			self.ContextMenu(mouse)
		elif mouse.button == 1:
			# check to see if the user clicked on the cancel button
			if self.cancelButtonArea.x <= mouse.x <= self.cancelButtonArea.width+self.cancelButtonArea.x \
				and self.cancelButtonArea.y <= mouse.y <= self.cancelButtonArea.height+self.cancelButtonArea.y \
				and self.event.isLoading:
				self.OnDelete()
				return True
			
			if 'GDK_SHIFT_MASK' in mouse.state.value_names:
				# LMB+shift: remove any existing selection in this event, begin 
				#   selecting part of this event
				self.isSelecting = True
				self.event.selection[0] = self.SecFromPixX(mouse.x)
				self.fadeMarkers = [100,100]
				if not self.selmessageID: 
					self.selmessageID = self.mainview.SetStatusBar(_("<b>Click</b> the buttons below the selection to do something to that portion of audio."))
			else:
				if self.fadeMarkersContext and self.fadeMarkersContext.in_fill(mouse.x, mouse.y):
					# LMB over a fadeMarker: drag that marker
					self.isDraggingFade = True
					if mouse.x > self.PixXFromSec(self.event.selection[1]) - self._PIXX_FADEMARKER_WIDTH - 1:
						self.fadeBeingDragged = 1
						return True
					else:
						self.fadeBeingDragged = 0
						return True
				if mouse.type == gtk.gdk._2BUTTON_PRESS:
					# LMB double-click: split here
					self.mouseAnchor[0] = mouse.x
					if self.event.isLoading == False:
						self.OnSplit(None, mouse.x)
					return True
				
				# remove any existing selection in this event
				self.event.selection = [0,0]
				if self.drawer.parent == self.lane.fixed:
					self.lane.fixed.remove(self.drawer)
					if self.volmessageID:   #clesr status bar if not already clear
						self.mainview.ClearStatusBar(self.volmessageID)
						self.volmessageID = None
					if self.selmessageID:   #clesr status bar if not already clear
						self.mainview.ClearStatusBar(self.selmessageID)
						self.selmessageID = None	
				self.isDragging = True
				
				self.eventStart = self.event.start
				ptr = gtk.gdk.display_get_default().get_pointer()
				self.mouseAnchor = [ptr[1], ptr[2]]
	
		return True

	#_____________________________________________________________________

	def OnFocus(self, widget, event):
		"""
			Select the event when focused via a non-mouse based method.
			
			Parameters:
				widget -- reserved for GTK callbacks, don't use it explicitly.
				event -- GTK focus event that fired this method call.
				
			Returns:
				True -- continue GTK signal propagation after processing event.
				False -- pass this event on to other handlers because we don't want it.
		"""

		self.queue_draw()

		return True

	#_____________________________________________________________________

	def OnFocusLost(self, widget, event):
		"""
			Deselect the event when focus is lost.
			
			Parameters:
				widget -- reserved for GTK callbacks, don't use it explicitly.
				event -- GTK focus event that fired this method call.
				
			Returns:
				True -- continue GTK signal propagation after processing the event.
		"""

		self.queue_draw()

		return True
	
	#_____________________________________________________________________

	def OnKeyPress(self, widget, event):
		"""
			Handle manipulation of events via the keyboard.

			Parameters:
				widget -- reserved for GTK callbacks, don't use it explicitly.
				event -- GTK keyboard event that fired this method call.
				
			Returns:
				True -- continue GTK signal propagation after processing the event. 
				False -- pass this event on to other handlers because we don't want it.
		"""

		#Don't allow moving, etc while recording!
		if self.event.instrument.project.GetIsRecording():
			return False

		modifier = 0.1 # Multiply movement by this amount (modified by ctrl key)
		moveCursor = False # Are we moving the highlight cursor or the event?
		moveLeftFade = False
		moveRightFade = False
		moveTo = None

		if "GDK_SHIFT_MASK" in event.state.value_names:
			if self.event.selection != [0, 0]:
				moveLeftFade = True
			moveCursor = True
			modifier = 0.5
		if "GDK_CONTROL_MASK" in event.state.value_names:
			if self.event.selection != [0, 0]:
				moveRightFade = True
			modifier *= 10
		if "GDK_MOD1_MASK" in event.state.value_names:
			moveCursor = True
			modifier *= 10
			if not self.isSelecting:
				if not self.highlightCursor:
					self.event.selection[0] = 0
				else:
					self.event.selection[0] = self.SecFromPixX(self.highlightCursor)
			self.UpdateFadeMarkers()
			self.isSelecting = True
			selection_direction = "ltor"
			selection = self.event.selection
			if selection[0] > selection[1]:
				selection_direction = "rtol"
				self.fadeMarkers.reverse()

		
		key = gtk.gdk.keyval_name(event.keyval)

		if key == "Return":
			# Toggle if this event is selected or not
			self.event.SetSelected(not self.event.isSelected)

			# Clear any selection that has been made
			self.event.selection = [0, 0]
			self.isSelecting = False
			self.highlightCursor = None
			self.HideDrawer()

			return True

		if not self.event.isSelected:
			# If this event isn't selected don't process any key events for it (except
			return False

		if key == "Up":
			# Adjust fade points
			if moveLeftFade:
				self.fadeMarkers[0] += 1
			if moveRightFade:
				self.fadeMarkers[1] += 1
			if self.fadeMarkers[0] > 100:
				self.fadeMarkers[0] = 100
			if self.fadeMarkers[1] > 100:
				self.fadeMarkers[1] = 100
			if moveLeftFade or moveRightFade:
				self.SetAudioFadePointsFromCurrentSelection()
		elif key == "Down":
			# Adjust fade points
			if moveLeftFade:
				self.fadeMarkers[0] -= 1
			if moveRightFade:
				self.fadeMarkers[1] -= 1
			if self.fadeMarkers[0] < 0:
				self.fadeMarkers[0] = 0
			if self.fadeMarkers[1] < 0:
				self.fadeMarkers[1] = 0
			if moveLeftFade or moveRightFade:
				self.SetAudioFadePointsFromCurrentSelection()
		elif key == "Left":
			# Move event/highlight cursor
			if not self.isSelecting:
				# Reset selection
				self.event.selection = [0, 0]
				self.fadeMarkers = [100, 100]
			if not self.highlightCursor:
				self.event.select = [0, 0]
				self.fadeMarkers = [100, 100]
				self.highlightCursor = 0
			if moveCursor:
				moveTo = self.highlightCursor - modifier
			else:
				moveTo = self.event.start - modifier
		elif key == "Right":
			# Move event/highlight cursor
			if not self.isSelecting:
				# Reset selection
				self.event.selection = [0, 0]
				self.fadeMarkers = [100, 100]
			if not self.highlightCursor:
				self.highlightCursor = 0
			if moveCursor:
				moveTo = self.highlightCursor + modifier
			else:
				moveTo = self.event.start + modifier
		elif key == "space" and not self.event.isLoading:
			if self.highlightCursor:
				# If we've got the highlight cursor out cut at that point
				self.OnSplit(None, self.highlightCursor)
			else:
				# Otherwise, stop playing and cut at the play position (if it's over this event)
				play_pos = self.project.transport.GetPixelPosition(self.event.start)
				if play_pos > 0 and play_pos < self.allocation.width:
					self.project.Stop()
					self.OnSplit(None, play_pos)
			return True
		else:
			return False

		if moveTo:
			# Don't go beyond respective boundaries
			if moveTo < 0:
				moveTo = 0
			if (moveCursor or self.isSelecting) and moveTo > self.allocation.width:
				moveTo = self.allocation.width

			if moveCursor:
				self.highlightCursor = moveTo
			else:
				self.event.MoveButDoNotOverlap(moveTo)
			if self.isSelecting:
				self.event.selection[1] = self.SecFromPixX(moveTo)

		# Hide the drawer if the selection has been cleared
		if self.event.selection == [0, 0]:
			self.HideDrawer()

		self.lane.UpdatePosition(self)
			
		return True

	#_____________________________________________________________________

	def OnKeyRelease(self, widget, event):
		"""
			Handle releasing of ALT key to stop drawing selection.
			
			Parameters:
				widget -- reserved for GTK callbacks, don't use it explicitly.
				event -- GTK keyboard event that fired this method call.
				
			Returns:
				True -- continue GTK signal propagation after processing the event.
				False -- pass this event on to other handlers because we don't want it.
		"""

		key = gtk.gdk.keyval_name(event.keyval)
		if self.isSelecting and "GDK_MOD1_MASK" in event.state.value_names and (key == "Left" or key == "Right"):
			self.isSelecting = False
			self.highlightCursor = None
			self.ShowDrawer()

		# Allow this even to be processed by other widgets regardless (so accelerators still work)
		return False

	#_____________________________________________________________________
	
	def ContextMenu(self, mouse):
		"""
		Creates a context menu in response to a right click.
		
		Parameters:
			mouse -- GTK mouse event that fired this method call.
		"""
		menu = gtk.Menu()
		
		splitImg = gtk.Image()
		splitImg.set_from_file(os.path.join(Globals.IMAGE_PATH, "icon_split.png"))
		items = [	(_("_Split"), self.OnSplit, True, splitImg, mouse.x),
					("---", None, None, None, None),
					(_("Cu_t"), self.OnCut, True, gtk.image_new_from_stock(gtk.STOCK_CUT, gtk.ICON_SIZE_MENU), None),
					(_("_Copy"), self.OnCopy, True, gtk.image_new_from_stock(gtk.STOCK_COPY, gtk.ICON_SIZE_MENU), None),
					(_("_Delete"), self.OnDelete, False, gtk.image_new_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_MENU), None)
					] 

		for label, callback, sometimes, image, param in items: 
			if label == "---":
				menuItem = gtk.SeparatorMenuItem()
			elif image:
				menuItem = gtk.ImageMenuItem(label, True)
				menuItem.set_image(image)
			else:
				menuItem = gtk.MenuItem(label=label)
			
			if self.event.isLoading and sometimes:
				menuItem.set_sensitive(False)
			else:
				menuItem.set_sensitive(True)
			menuItem.show() 
			menu.append(menuItem) 
			if callback:
				if param:
					menuItem.connect("activate", callback, param) 
				else:
					menuItem.connect("activate", callback)
		self.highlightCursor = mouse.x
		self.popupIsActive = True

		menu.popup(None, None, None, mouse.button, mouse.time)
		menu.connect("selection-done",self.OnMenuDone)
		
		self.mouseAnchor = [mouse.x, mouse.y]
			
	#_____________________________________________________________________
	
	def OnMenuDone(self, widget):
		"""
		Hides the right-click context menu after the user has selected one
		of its options or clicked elsewhere.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		self.popupIsActive = False
		self.highlightCursor = None
		
	#_____________________________________________________________________
		
	def OnMouseUp(self, widget, mouse):
		"""
		Called when the left mouse button is released.
		Finishes drag, fade and selection operations.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- GTK mouse event that fired this method call.
		"""
		if mouse.button == 1:
			if self.isDragging:		
				self.isDragging = False
				if (self.eventStart != self.event.start):
					self.event.Move(self.event.start, self.eventStart)
					return False #need to pass this button release up to RecordingView
			elif self.isDraggingFade:
				self.isDraggingFade = False
				# set the audioFadePoints appropriately
				self.SetAudioFadePointsFromCurrentSelection()
			elif self.isSelecting:
				self.isSelecting = False
				self.ShowDrawer()

	#_____________________________________________________________________

	def ShowDrawer(self):
		"""
		Cause the draw to be shown at the current event selection position.
		"""
		selection_direction = "ltor"
		selection = self.event.selection
		if selection[0] > selection[1]:
			self.event.selection = [selection[1], selection[0]]
			selection_direction = "rtol"
		
		#set the drawer align position
		self.drawerAlignToLeft = (selection_direction == "rtol")
		#move the drawer to its proper position
		self.UpdateDrawerPosition()
		
	#_____________________________________________________________________
		
	def OnMouseLeave(self, widget, event):
		"""
		Clears the StatusBar message when the mouse moves out of the
		EventLaneViewer area. It also disables cursors accordingly.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- GTK mouse event that fired this method call.
		"""
		if self.messageID:   #clear status bar if not already clear
			self.mainview.ClearStatusBar(self.messageID)
			self.messageID = None
		self.highlightCursor = None
		self.queue_draw()
		
	#_____________________________________________________________________
			
	def OnSplit(self, gtkevent, pos):
		"""
		Splits an Event in two.
		
		Parameters:
			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
			position -- The position in the event to split
		"""
		if self.event.selection != [0,0]:
			undoAction = self.project.NewAtomicUndoAction()
			self.event.SplitEvent(self.event.selection[1], _undoAction_=undoAction)
			self.event.SplitEvent(self.event.selection[0], _undoAction_=undoAction)
			self.event.selection = [0,0]
			self.HideDrawer()
		else:
			if pos == 0.0:
				return
			else:
				pos /= float(self.project.viewScale)
				self.event.SplitEvent(pos)
		
	#_____________________________________________________________________
	
	def OnCut(self, gtkevent):
		"""
		Cuts the selected portion of the Event, and puts it on the clipboard.
		
		Parameters:
			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.event.selection != [0,0]:
			undoAction = self.project.NewAtomicUndoAction()
			self.event.SplitEvent(self.event.selection[1], _undoAction_=undoAction)
			e = self.event.SplitEvent(self.event.selection[0], _undoAction_=undoAction)
			self.project.clipboardList = [e]
			e.Delete(_undoAction_=undoAction)
			self.event.selection = [0,0]
			self.HideDrawer()
		else:
			self.project.clipboardList = [self.event]
			self.OnDelete()
	
	#_____________________________________________________________________
	
	def OnCopy(self, gtkevent):
		"""
		Copies the selected portion of the Event to the clipboard.
		
		Parameters:
			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.event.selection != [0,0]:
			e = self.event.CopySelection()
			self.project.clipboardList = [e]
			#We shouldn't hide the drawer here, unfriendly behaviour
		else:
			self.project.clipboardList = [self.event]
	
	#_____________________________________________________________________

	def OnDelete(self, event=None):
		"""
		Called when "Delete" is selected from context menu.
		Deletes the selected Event from the Project.
			
		Parameters:
			event -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.event.selection != [0,0]:
			undoAction = self.project.NewAtomicUndoAction()
			self.event.SplitEvent(self.event.selection[1], _undoAction_=undoAction)
			e = self.event.SplitEvent(self.event.selection[0], _undoAction_=undoAction)
			e.Delete(_undoAction_=undoAction)
			self.event.selection = [0,0]
			self.HideDrawer()
		else:
			self.event.Delete()
	
	#_____________________________________________________________________
		
	def TrimToSelection(self, gtkevent):
		"""
		Cut this Event down so only the selected bit remains. This Event
		is L-S-R, where S is the selected bit; L and R will be removed.
		
		Parameters:
			gtkevent -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.event.isLoading == True:
			return

		self.HideDrawer()

		self.event.Trim(self.event.selection[0], self.event.selection[1])
		self.event.selection = [0,0]

	#_____________________________________________________________________

	def HideDrawer(self):
		"""
		Hide the drawer.
		"""
		
		self.lane.RemoveDrawer(self.drawer)

	#_____________________________________________________________________
	
	def do_size_request(self, requisition):
		"""
		This function has been overrided to avoid getting a 1x1 display size.
		
		Parameters:
			requisition -- TODO
		"""
		if self.event.duration > 0:
			requisition.width = self.event.duration * self.project.viewScale
		elif self.event.loadingLength > 0:
			requisition.width = self.event.loadingLength * self.project.viewScale
		else:
			requisition.width = 1 * self.project.viewScale
			
		if not (self.small):
			requisition.height = 77
		else:
			requisition.height = 30
#			rect = self.get_allocation()
#			
#			if rect.height < 30:
#				requisition.height = 30
#			else:
#				requisition.height = rect.height

	#_____________________________________________________________________
	
	def OnEventPosition(self, event):
		"""
		Callback function for when the position of the event changes.
		"""
		self.SetAccessibleName()
		self.lane.UpdatePosition(self)
	
	#_____________________________________________________________________
	
	def OnEventWaveform(self, event):
		"""
		Callback function for when the waveform of the event changes.
		"""
		self.redrawWaveform = True
		self.UpdateFadeMarkers()
		self.queue_draw()
		
	#_____________________________________________________________________
	
	def OnEventLoading(self, event):
		"""
		Callback function for when the loading status of the event changes.
		"""
		self.drawer.set_sensitive(not self.event.isLoading)
		self.queue_draw()
		
	#_____________________________________________________________________
	
	def OnEventLength(self, event):
		"""
		Callback function for when the length of the event changes.
		"""
		self.redrawWaveform = True
		self.SetAccessibleName()
		self.queue_resize()
		self.queue_draw()
		
	#_____________________________________________________________________
	
	def OnEventSelected(self, event):
		"""
		Callback function for when the event is selected or de-selected.
		"""
		self.queue_draw()
		
	#_____________________________________________________________________
	
	def OnEventCorrupt(self, event, error):
		"""
		Callback function for when the event's file is found to be corrupt.
		"""
		message="%s %s\n\n%s" % (_("Error loading file:"), self.event.filelabel, 
					_("Please make sure the file exists, and the appropriate plugin is installed."))
			
		outputtext = "\n\n".join((message, error))
		
		dlg = gtk.MessageDialog(None,
			gtk.DIALOG_MODAL,
			gtk.MESSAGE_ERROR,
			gtk.BUTTONS_CLOSE,
			outputtext)
		dlg.connect('response', lambda dlg, response: dlg.destroy())
		dlg.show()
		self.OnDelete()
		
	#_____________________________________________________________________
	
	def OnProjectZoom(self, project):
		"""
		Callback for when the zoom level of the project changes.
		
		Parameters:
			project -- The project instance that send the signal.
		"""
		if self.currentScale != self.project.viewScale:
			self.redrawWaveform = True
			self.queue_resize()
			self.currentScale = self.project.viewScale
			self.queue_draw()

	#_____________________________________________________________________

	def PixXFromSec(self, sec):
		"""
		Converts seconds to an X pixel position in the waveform.
		
		Parameters:
			sec -- value in seconds.
			
		Returns:
			the correspondent pixel X position in the waveform.
		"""
		return round(float(sec) * self.project.viewScale)
	
	#_____________________________________________________________________
	
	def SecFromPixX(self, pixx):
		"""
		Converts an X pixel position in the waveform into seconds.
		
		Parameters:
			pixx -- X pixel position value.
			
		Returns:
			the correspondent value in seconds.
		"""
		return float(pixx) / self.project.viewScale
	
	#_____________________________________________________________________
	
	def PixYFromVol(self, vol):
		"""
		Converts a volume value into a Y pixel position in the waveform.
		
		Parameters:
			vol -- volume value in a [0.0, 1.0] range.
			
		Returns:
			the correspondent pixel Y position in the waveform.
		"""
		return round((1.0 - vol) * self.allocation.height)
	
	#_____________________________________________________________________
	
	def VolFromPixY(self, pixy):
		"""
		Converts a Y pixel position in the waveform into a volume value.
		
		Parameters:
			pixy -- Y pixel position value.
			
		Returns:
			the correspondent value, in seconds, in a [0.0, 1.0] range.
		"""
		return 1.0 - (float(pixy) / self.allocation.height)

	#_____________________________________________________________________
	
	def SetAudioFadePointsFromCurrentSelection(self):
		"""
		Creates fade points for the current selection.
		"""
		volLeft = self.fadeMarkers[0] / 100.0
		volRight = self.fadeMarkers[1] / 100.0
		
		selection = self.event.selection
		self.event.AddAudioFadePoints(selection[0], selection[1], volLeft, volRight)
	
	#_____________________________________________________________________
	
	def GetSelectionAsPixels(self):
		"""
		Obtain the Event selection as a list of two points, measured in
		pixels instead of seconds like Event.selection.
		
		Returns:
			list with two X points describing the selection.
		"""
		x1 = self.PixXFromSec(self.event.selection[0])
		x2 = self.PixXFromSec(self.event.selection[1])
		return [x1, x2]
	
	#_____________________________________________________________________
	
	def UpdateDrawerPosition(self, reverseSelectionPoints=False):
		"""
		Updates the drawer position to the correct position when user
		moves the mouse over the EventViewer.
		
		Parameters:
			reverseSelectionPoints -- True if the selection points should
										be reversed.
		"""
		
		if reverseSelectionPoints:
			selection = [self.event.selection[1], self.event.selection[0]]
		else:
			selection = self.event.selection[:]

		x0 = self.project.viewScale * self.event.selection[0]	
		x1 = (self.project.viewScale * self.event.selection[1]) - x0

		if x0 < self.drawer.size_request()[0] and x1< self.drawer.size_request()[0]:
			self.drawerAlignToLeft = True

		eventx = int((self.event.start - self.project.viewStart) * self.project.viewScale)
		if self.drawerAlignToLeft:
			x = int(self.PixXFromSec(selection[0]))
		else:
			width = self.drawer.allocation.width
			if width == 1:
				width = 40 # fudge it because it has no width initially
			x = int(self.PixXFromSec(selection[1]) - width)
		
		self.lane.PutDrawer(self.drawer, eventx + x)
		#don't update the lane because it calls us and that might cause infinite loop

	#_____________________________________________________________________
	
	def DeleteSelectedFadePoints(self, event):
		"""
		Deletes the selected fade points from the Event.
		
		Parameters:
			event -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.event.isLoading == True:
			return
		self.event.DeleteSelectedFadePoints()
		
	#_____________________________________________________________________
	
	def SnapSelectionToFadePoints(self, event):
		"""
		Snaps the selection to a set of fade points.
		
		Parameters:
			event -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if len(self.event.audioFadePoints) < 2:
			#not enough levels
			return
			
		points = [x[0] for x in self.event.audioFadePoints]
		left, right = self.event.selection
		
		leftOfLeft = max([x for x in points if x < left])
		rightOfLeft = min([x for x in points if x >= left])
		
		leftOfRight = max([x for x in points if x < right])
		rightOfRight = min([x for x in points if x >= right])
		
		if abs(leftOfLeft - left) < abs(rightOfLeft - left):
			leftChooses = leftOfLeft
		else:
			leftChooses = rightOfLeft

		if abs(leftOfRight - right) > abs(rightOfRight - right):
			rightChooses = rightOfRight
		else:
			rightChooses = leftOfRight

		if leftChooses == rightChooses:
			#the both selected the same point
			if abs(leftChooses - left) > abs(rightChooses - right):
				#if right is closer to the point
				leftChooses = leftOfLeft
			else:
				rightChooses = rightOfRight
		
		self.event.selection = [leftChooses, rightChooses]
		self.UpdateFadeMarkers()
		self.queue_draw()
		
	#_____________________________________________________________________
	
	def UpdateFadeMarkers(self):
		"""
		Called when the a fade point's value changes, to update the graphical
		marker over the waveform.
		"""
		self.fadeMarkers = [self.event.GetFadeLevelAtPoint(x) * 100 for x in self.event.selection]
	
	#_____________________________________________________________________

	def SetAccessibleName(self):
		"""
		Set an ATK name to help users with screenreaders.
		"""
		accessible = self.get_accessible()
		accessible.set_name(_("Event, %(name)s, %(dur)0.2f seconds long, starting at %(start)0.2f seconds.") \
				% {"name":self.event.name, "dur":self.event.duration, "start":self.event.start})
		
	#_____________________________________________________________________

	def ChangeSize(self, small):
		"""
		Changes size of event viewer.
		
		Parameters:
			small -- True if the event viewer is to change to small
		"""
		self.small = small
		self.queue_resize()
		self.queue_draw()
			
	#____________________________________________________________________	


#=========================================================================



import Jokosher.Extension
import gobject
import dbus
import dbus.service
#if we are using the old version of dbus, we need dbus's glib too.
if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
	import dbus.glib

# Extension meta information
EXTENSION_NAME = "Jokosher DBus API"
EXTENSION_DESCRIPTION = "Allows other processes to call Jokosher extension API functions using DBus"
EXTENSION_VERSION = "0.1"

# Extension constants
JOKOSHER_DBUS_PATH = "/org/gnome/Jokosher"
JOKOSHER_DBUS_NAME = "org.gnome.Jokosher"

class HelloWorldObject(dbus.service.Object):
	def __init__(self, bus_name, object_path=JOKOSHER_DBUS_PATH):
		dbus.service.Object.__init__(self, bus_name, object_path)

	## DBus API Methods ##
	#_______________________________________
	@dbus.service.method(JOKOSHER_DBUS_NAME)
	def play(self):
		API.play()
	
	@dbus.service.method(JOKOSHER_DBUS_NAME)	
	def stop(self):
		API.stop()
		
	## DBus API Signals ##
	#_______________________________________
	@dbus.service.signal(JOKOSHER_DBUS_NAME)
	def signal_play(self, message):
		pass	
	
	@dbus.service.signal(JOKOSHER_DBUS_NAME)
	def signal_stop(self, message):
		pass


#initialize the extension
def startup(api):
	global API, dbusObject
	API = api
	session_bus = dbus.SessionBus()
	bus_name = dbus.service.BusName('org.gnome.Jokosher', bus=session_bus)
	dbusObject = HelloWorldObject(bus_name)

#disable/shutdown the extension
def shutdown():
	#TODO, remove the org.gnome.Jokosher dbus service
	pass


#
#	THIS FILE IS PART OF THE JOKOSHER PROJECT AND LICENSED UNDER THE GPL. SEE
#	THE 'COPYING' FILE FOR DETAILS
#
#	VUWidget.py
#	
#	This module draws the gradient volume levels and is used by
#	MixerStrip.py to show the volume levels in Jokosher's mix view.
#
#-------------------------------------------------------------------------------

import pygtk
pygtk.require("2.0")
import gtk
import cairo

import gettext
_ = gettext.gettext

#=========================================================================

class VUWidget(gtk.DrawingArea):
	"""
	Draws the gradient volume levels and is used by MixerStrip.py
	to show the volume levels in Jokosher's mix view.
	"""
	
	""" GTK widget name """
	__gtype_name__ = 'VUWidget'
	
	""" Height, in pixels, for the volume handle """
	_VH_HEIGHT = 25
	""" Space between the edge of the volume handle and the edge of the VU meter"""
	_VH_PADDING = 20
	_VH_BORDER_WIDTH = 5
	""" the highest value allowed to be set for the volume (lowest is always zero)"""
	_MAX_VOLUME = 2
	"""the amount to move the volume up or down by when scrolling or pressing the arrow keys"""
	_VOLUME_STEP_AMOUNT = 0.2
	
	""" Both text height and width below depend on the font size. 
	If you change the font size, figure out the new pixel sizes and update these values."""
	_FONT_SIZE = 18
	_TEXT_HEIGHT = 13
	_TEXT_WIDTH = 37
	
	"""
	Various color configurations:
	   ORGBA = Offset, Red, Green, Blue, Alpha
	   RGBA = Red, Green, Blue, Alpha
	   RGB = Red, Green, Blue
	"""
	_VH_ACTIVE_RGBA = (1., 1., 1., 1.)
	_VH_INACTIVE_RGBA = (1., 1., 1., 0.75)
	_VH_BORDER_RGBA = (0.5, 0., 0., 0.75)
	_BACKGROUND_RGB = (0., 0., 0.)
	_LEVEL_GRADIENT_BOTTOM_ORGBA = (1, 0, 1, 0, 1)
	_LEVEL_GRADIENT_TOP_ORGBA = (0, 1, 0, 0, 1)
	_TEXT_RGBA = (0., 0., 0., 1.)
	
	"""
	   The events we wish to receive after we grab the mouse (and it is no longer above this widget)
	   If events other than mouse event are put in here, it may cause the program to crash.
	"""
	_POINTER_GRAB_EVENTS = (
			gtk.gdk.POINTER_MOTION_MASK |
			gtk.gdk.BUTTON_RELEASE_MASK |
			gtk.gdk.BUTTON_PRESS_MASK |
			gtk.gdk.LEAVE_NOTIFY_MASK )
	
	#_____________________________________________________________________
	
	def __init__(self, mixerstrip, mainview):
		"""
		Creates a new instance of VUWidget.
		
		Parameters:
			mixerstrip -- TODO
			mainview -- the main Jokosher window (JokosherApp).
		"""
		gtk.DrawingArea.__init__(self)
		self.mixerstrip = mixerstrip
		self.mainview = mainview
		
		self.set_events(self._POINTER_GRAB_EVENTS | 
				gtk.gdk.KEY_PRESS_MASK |
				gtk.gdk.SCROLL_MASK )
		
		self.set_flags(gtk.CAN_FOCUS)
		
		self.connect("button-press-event", self.OnMouseDown)
		self.connect("button-release-event", self.OnMouseUp)
		self.connect("motion_notify_event", self.OnMouseMove)
		self.connect("leave_notify_event", self.OnMouseLeave)
		self.connect("configure_event", self.OnSizeChanged)
		self.connect("expose-event", self.OnDraw)
		self.connect("key_press_event", self.OnKeyPress)
		self.connect("scroll-event", self.OnScroll)
		
				
		self.fader_active = False
		self.fader_hover = False
		self.message_id = None
		
		self.source = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.allocation.width, self.allocation.height)
		
	#_____________________________________________________________________
		
	def OnMouseDown(self, widget, mouse):
		"""
		If the fader widget is clicked, activates it.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		#If the slider is right-clicked then set the volume to 100%
		if mouse.button == 3:
			self.mixerstrip.SetVolume(1.0)
			self.queue_draw()
			return True

		if self.__YPosOverVolumeHandle(mouse.y):
			response = gtk.gdk.pointer_grab(self.window, False, self._POINTER_GRAB_EVENTS, None, None, mouse.time)
			self.fader_active = (response == gtk.gdk.GRAB_SUCCESS)
			self.queue_draw()
		
		self.grab_focus()
		
		#Returning True is very important!! It tells other not to handle the mouse event for us.
		#Without it weird thing will happen; pointer grabs will not work, and button-release-events will sometimes not be sent, etc.
		return True
	
	#_____________________________________________________________________
	
	def OnMouseMove(self, widget, mouse):
		"""
		Displays a helper message in the StatusBar and sets the volume
		according to the position of the fader widget.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if not self.message_id:
			self.message_id = self.mainview.SetStatusBar(_("<b>Drag</b> the <b>slider</b> to alter volume levels - <b>Right-Click</b> to set the <b>slider</b> to 100%"))
		
		oldHover = self.fader_hover
		self.fader_hover = (0 < mouse.x < self.get_allocation().width) and self.__YPosOverVolumeHandle(mouse.y)
		
		if self.fader_active:
			rect = self.get_allocation()
			volume = 1. - ((mouse.y - self._VH_HEIGHT/2) /  (rect.height-self._VH_HEIGHT))
			volume  = max(volume, 0.)
			volume  = min(volume, 1.)
			self.mixerstrip.SetVolume(volume * self._MAX_VOLUME)
		
		if self.fader_active or oldHover != self.fader_hover:
			self.queue_draw()
		
		return True
	
	#_____________________________________________________________________
	
	def OnMouseUp(self, widget, mouse):
		"""
		Deactivates the fader widget.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.fader_active:
			self.fader_active = False
			gtk.gdk.pointer_ungrab(mouse.time)
			self.queue_draw()
			self.mixerstrip.CommitVolume()
		
		return True
		
	#_____________________________________________________________________
	
	def OnMouseLeave(self, widget, mouse):
		"""
		Clears the StatusBar helper message.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.message_id:
			self.mainview.ClearStatusBar(self.message_id)
			self.message_id = None
		
		if self.fader_hover:
			self.fader_hover = False
			self.queue_draw()
			
		return True

	#_____________________________________________________________________
	
	def OnKeyPress(self, widget, event):
		"""
		Moves the volume up or down based on the key presses.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		
		key = gtk.gdk.keyval_name(event.keyval)
		if key == "Up" or key == "Down":
			self.ChangeVolumeByStep(increaseVolume=(key == "Up"))
			return True
		return False
		
	#_____________________________________________________________________
	
	def OnScroll(self, widget, event):
		"""
		Moves the volume up or down based on the scroll motion.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.fader_active:
			return False
		
		if event.direction == gtk.gdk.SCROLL_UP or event.direction == gtk.gdk.SCROLL_DOWN:
			self.ChangeVolumeByStep(increaseVolume=(event.direction == gtk.gdk.SCROLL_UP))
			#Don't make the handle glow if it has moved out from under the mouse
			self.fader_hover = self.fader_hover = (0 < event.x < self.get_allocation().width) and self.__YPosOverVolumeHandle(event.y)
			return True
		
		return False
	
	#_____________________________________________________________________
	
	def ChangeVolumeByStep(self, increaseVolume):
		"""
		Moves the volume up or down based on the volume step size.
		
		Parameters:
			increaseVolume -- if True, volume will be increased, otherwise it will be decreased.
		"""
		if increaseVolume:
			volume = self.mixerstrip.GetVolume() + self._VOLUME_STEP_AMOUNT
		else:
			volume = self.mixerstrip.GetVolume() - self._VOLUME_STEP_AMOUNT
		
		volume = max(0, min(self._MAX_VOLUME, volume))
		self.mixerstrip.SetVolume(volume)
		
		self.queue_draw()
		
	#_____________________________________________________________________

	def OnSizeChanged(self, obj, evt):
		"""
		Toggles a redraw of the VUWidget if needed.
		
		Parameters:
			obj -- reserved for Cairo callbacks, don't use it explicitly. *CHECK*
			evt --reserved for Cairo callbacks, don't use it explicitly. *CHECK*
		"""
		if self.allocation.width != self.source.get_width() or self.allocation.height != self.source.get_height():
			self.source = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.allocation.width, self.allocation.height)
			self.GenerateBackground()

	#_____________________________________________________________________

	def GenerateBackground(self):
		"""
		Renders the gradient strip for the VU meter background to speed up drawing.
		"""
		
		rect = self.get_allocation()

		ctx = cairo.Context(self.source)
		ctx.set_line_width(2)
		ctx.set_antialias(cairo.ANTIALIAS_SUBPIXEL)
		
		# Create our green to red gradient
		pat = cairo.LinearGradient(0.0, 0.0, 0, rect.height)
		pat.add_color_stop_rgba(*self._LEVEL_GRADIENT_BOTTOM_ORGBA)
		pat.add_color_stop_rgba(*self._LEVEL_GRADIENT_TOP_ORGBA)

		# Fill the volume bar
		ctx.rectangle(0, 0, rect.width, rect.height)
		ctx.set_source(pat)
		ctx.fill()
	#_____________________________________________________________________

	def OnDraw(self, widget, event):
		"""
		Handles the GTK paint event.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			event -- reserved for GTK callbacks, don't use it explicitly.
			
		Returns:
			False -- TODO
		"""
		
		ctx = widget.window.cairo_create()
		
		rect = self.get_allocation()

		# Fill a black background		
		ctx.rectangle(0, 0, rect.width, rect.height)
		ctx.set_source_rgb(*self._BACKGROUND_RGB)
		ctx.fill()

		# Blit across the cached gradient backgound
		ctx.rectangle(0, rect.height * (1. - self.mixerstrip.GetLevel()), rect.width, rect.height)
		ctx.clip()
		ctx.set_source_surface(self.source, 0, 0)	
		ctx.paint()

		# Reset the clip region
		ctx.reset_clip()
		
		# Draw the current volume level bar, with highlight if appropriate
		vpos = self.__GetVolumeHandleYPos()
		if self.fader_active:
			ctx.set_source_rgba(*self._VH_BORDER_RGBA)
			ctx.set_line_width(self._VH_HEIGHT + self._VH_BORDER_WIDTH)
			ctx.set_line_cap(cairo.LINE_CAP_ROUND)
			ctx.move_to(self._VH_PADDING, vpos)
			ctx.line_to(rect.width - self._VH_PADDING, vpos)
			ctx.stroke()
			ctx.set_source_rgba(*self._VH_ACTIVE_RGBA)
		elif self.fader_hover:
			ctx.set_source_rgba(*self._VH_ACTIVE_RGBA)
		else:
			ctx.set_source_rgba(*self._VH_INACTIVE_RGBA)

		ctx.set_line_cap(cairo.LINE_CAP_ROUND)
		ctx.set_line_width(self._VH_HEIGHT)
		ctx.move_to(self._VH_PADDING, vpos)
		ctx.line_to(rect.width - self._VH_PADDING, vpos)
		ctx.stroke()

		# Draw the volume level in the bar
		ctx.set_source_rgba(*self._TEXT_RGBA)
		textYOffset = (self._VH_HEIGHT - self._TEXT_HEIGHT) / 2
		textXOffset = ((rect.width - (2 * self._VH_PADDING) - self._TEXT_WIDTH) / 2) + self._VH_PADDING
		ctx.move_to(textXOffset, vpos + textYOffset)
		ctx.set_font_size(self._FONT_SIZE)
		text = "%d%%" % (self.mixerstrip.GetVolume() * 100)
		ctx.show_text(text)

		return False
		
	#_____________________________________________________________________
	
	def do_size_request(self, requisition):
		"""
		TODO
		
		Parameters:
			requisition -- TODO
		"""
		requisition.width = 100
		requisition.height = -1
		
	#_____________________________________________________________________
	
	def Destroy(self):
		"""
		Deletes the cairo.ImageSurface and then calls the class destructor.
		"""
		del self.source
		self.destroy()
	
	#_____________________________________________________________________
	
	def __GetVolumeHandleYPos(self):
		"""
		Calculates the verical position of the *center* of the volume handle
		based on the instrument's current volume, and the size of the volume
		handle itself (so that is doesn't go off the screen at the top or bottom.
		
		Returns:
			the Y value in pixels of the center of the volume handle.
		"""
		height = self.get_allocation().height
		totalPixelHeight = height - self._VH_HEIGHT
		inverseVolume = (self._MAX_VOLUME - self.mixerstrip.GetVolume()) / self._MAX_VOLUME
		offset = self._VH_HEIGHT / 2
		
		return totalPixelHeight * inverseVolume + offset
	
	#_____________________________________________________________________
	
	def __YPosOverVolumeHandle(self, yPos):
		"""
		Calculates if the given vertical position is located over top of the volume handle.
		
		Parameters:
			yPos -- the vertical position in pixels
		
		Returns:
			True if the value is over the volume handle; False otherwise.
		"""
		handleYPos = self.__GetVolumeHandleYPos()
		halfHandle = self._VH_HEIGHT / 2
		return handleYPos - halfHandle < yPos < handleYPos + halfHandle
	
	#_____________________________________________________________________
#=========================================================================



#
#	THIS FILE IS PART OF THE JOKOSHER PROJECT AND LICENSED UNDER THE GPL. SEE
#	THE 'COPYING' FILE FOR DETAILS
#
#	RecordingView.py
#	
#	A sub-class of gtk.Frame containing the visual layout of instrument
#	tracks, timeline, and horizontal scrollbars.
#
#-------------------------------------------------------------------------------

import gtk
import InstrumentViewer
import TimeLineBar
import Globals
import ui.MessageArea as MessageArea
from gettext import gettext as _
import urllib

#=========================================================================

class RecordingView(gtk.Frame):
	"""
	This class encapsulates a visual layout of a project comprising
	instrument tracks, timeline, and horizontal scrollbars.
	"""
	
	""" GTK widget name """
	__gtype_name__ = 'RecordingView'
	
	""" How far in you are allowed to zoom. """
	ZOOM_MAX_SCALE = 100.0
	""" How for out you are allowed to zoom. 
	This is just a default value and will change 
	depending on the project length."""
	ZOOM_MIN_SCALE = 5.0
	
	""" Number only to be used inside Jokosher """
	URI_DRAG_TYPE = 86
	
	""" Custom numbers for use while dragging text in Jokosher """
	DRAG_TARGETS = [ ( "text/uri-list", 	# Accept uri-lists
						0,					# From everywhere
						URI_DRAG_TYPE ),		# Use the custom number
						("text/plain", 0, URI_DRAG_TYPE) # so drags from Firefox work
						]
	
	"""The number of seconds shown after the end of the last event"""
	EXTRA_SCROLL_TIME = 25

	#_____________________________________________________________________

	def __init__(self, project, mainview, small=False):
		"""
		Creates a new instance of RecordingView.
		
		Parameters:
			project -- the currently active Project.
			mainview -- the main Jokosher window (MainApp).
			small -- set to True if we want small edit views (i.e. for the mixing view).
		"""
		gtk.Frame.__init__(self)

		self.project = project
		self.mainview = mainview
		self.small = small
		self.timelinebar = TimeLineBar.TimeLineBar(self.project, mainview)
		
		self.errorMessageArea = None
		self.restoreMessageArea = None
		
		## create darker workspace box
		self.eventBox = gtk.EventBox()
		self.eventBox.connect("button-press-event", self.OnEmptySpaceDoubleClicked)
		
		self.vbox = gtk.VBox()
		self.add(self.vbox)
		self.vbox.pack_start(self.timelinebar, False, False)
		self.instrumentWindow = gtk.ScrolledWindow()
		self.instrumentBox = gtk.VBox()
		
		## pack the instrument box inside the eventbox as you cant modify the color of a gtk.Box
		self.eventBox.add(self.instrumentBox)
		self.instrumentWindow.add_with_viewport(self.eventBox)
		self.instrumentWindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		self.vbox.pack_start(self.instrumentWindow, True, True)
		self.instrumentWindow.child.set_shadow_type(gtk.SHADOW_NONE)
		self.views = []	
		
		self.header_size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
		self.header_size_group.add_widget(self.timelinebar.GetHeaderWidget())
		
		self.hb = gtk.HBox()
		self.hb.set_spacing(6)
		self.hb.set_border_width(6)
		self.vbox.pack_end(self.hb, False, False)
		self.vbox.pack_end(gtk.HSeparator(), False, False)
		
		self.zoom_hb = gtk.HBox()
		self.zoom_hb.set_spacing(6)
		self.zoom_hb.set_border_width(0)
		self.header_size_group.add_widget(self.zoom_hb)
		
		self.scrollRange = gtk.Adjustment()
		self.scrollBar = gtk.HScrollbar(self.scrollRange)
		
		self.hb.pack_start(self.zoom_hb, False, False)
		self.hb.pack_start(self.scrollBar, True, True)
		
		self.lastzoom = 0
		
		self.zoomSlider = gtk.HScale()
		self.zoomSlider.set_size_request(70, -1)
		
		self.zoomSlider.set_range(self.ZOOM_MIN_SCALE, self.ZOOM_MAX_SCALE)
		self.zoomSlider.set_increments(0.2, 0.2)
		self.zoomSlider.set_draw_value(False)
		self.zoomSlider.set_value(self.project.viewScale)
		self.zoomtip = gtk.Tooltips()
		self.zoomtip.set_tip(self.zoomSlider, _("Zoom the timeline - Right-Click to reset to the default level"), None)
		
		self.zoomSlider.connect("value-changed", self.OnZoom)
		self.zoomSlider.connect("button-press-event", self.OnZoomReset)
		
		self.inbutton = gtk.Button()
		inimg = gtk.image_new_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
		self.inbutton.set_image(inimg)
		self.inbutton.set_relief(gtk.RELIEF_NONE)
		self.zoomtip.set_tip(self.inbutton, _("Zoom in timeline"), None)
		self.inbutton.connect("clicked", self.OnZoomIn)
		
		self.outbutton = gtk.Button()
		outimg = gtk.image_new_from_stock(gtk.STOCK_ZOOM_OUT, gtk.ICON_SIZE_BUTTON)
		self.outbutton.set_image(outimg)
		self.outbutton.set_relief(gtk.RELIEF_NONE)
		self.zoomtip.set_tip(self.outbutton, _("Zoom out timeline"), None)
		self.outbutton.connect("clicked", self.OnZoomOut)

		self.zoom_hb.pack_start( self.outbutton, False, False)
		self.zoom_hb.pack_start( self.zoomSlider, False, False)
		self.zoom_hb.pack_start( self.inbutton, False, False)
		
		self.centreViewOnPosition = False
		self.scrollRange.lower = 0
		self.scrollRange.upper = 100
		self.scrollRange.value = 0
		self.scrollRange.step_increment = 1
		
		self.scrollBar.connect("value-changed", self.OnScroll)
		self.connect("expose-event", self.OnExpose)
		self.connect("button_release_event", self.OnExpose)
		self.connect("button_press_event", self.OnMouseDown)
		
		#connect to the project signals
		self.project.connect("gst-bus-error", self.OnProjectGstError)
		self.project.connect("incremental-save", self.OnProjectIncSave)
		self.project.connect("instrument::added", self.OnInstrumentAdded)
		self.project.connect("instrument::reordered", self.OnInstrumentReordered)
		self.project.connect("instrument::removed", self.OnInstrumentRemoved)
		self.project.connect("view-start", self.OnViewStartChanged)
		
		self.vbox.drag_dest_set(	gtk.DEST_DEFAULT_DROP,
									self.DRAG_TARGETS, 
									gtk.gdk.ACTION_COPY)
		self.vbox.connect("drag_data_received", self.OnDragDataReceived)
		self.vbox.connect("drag_motion", self.OnDragMotion)
		
		#add the instruments that were loaded from the project file already
		for instr in self.project.instruments:
			self.OnInstrumentAdded(project, instr)
			
		self.show_all()
		self.show_all()
		if self.small:
			self.inbutton.hide()
			self.outbutton.hide()
			self.zoomSlider.hide()
		
		if self.project.CanDoIncrementalRestore():
			message = _("Would you like to restore the current project?")
			details = _("A crash was detected and changes to your project were not saved.\nIf you would like, you can attempt to recover these lost changes.")
			
			msg_area = MessageArea.MessageArea()
			msg_area.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
			msg_area.add_stock_button_with_text(_("_Restore Project"), gtk.STOCK_APPLY, gtk.RESPONSE_OK)
			msg_area.set_text_and_icon(gtk.STOCK_DIALOG_QUESTION, message, details)
			
			msg_area.connect("response", self.OnRestoreMessageAreaResponse, msg_area)
			msg_area.connect("close", self.OnRestoreMessageAreaClose, msg_area)
			
			self.vbox.pack_end(msg_area, False, False)
			msg_area.show()
			self.restoreMessageArea = msg_area
		
	#_____________________________________________________________________

	def OnExpose(self, widget, event):
		"""
		Sets scrollbar properties (i.e. size, scroll increments, etc),
		once space for the object has been allocated.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			event -- reserved for GTK callbacks, don't use it explicitly.
		"""
		st = self.rc_get_style().bg[gtk.STATE_NORMAL]
		
		## Set the new color components
		st.red -= 3000
		st.green -= 3000
		st.blue -= 3000
		
		cmap = self.get_colormap()
		
		## allocate new colors
		col = cmap.alloc_color(st.red, st.green, st.blue)
		stcp = self.get_style().copy()
		stcp.bg[gtk.STATE_NORMAL] = col
		
		## use the new colormap
		self.eventBox.set_style(stcp)
		
		# calculate scrollable width (scroll bar should always be same width as viewable area)
		self.scrollRange.page_size = (self.scrollBar.allocation.width) / self.project.viewScale
		self.scrollRange.page_increment = self.scrollRange.page_size
		# add EXTRA_SCROLL_TIME extra seconds
		length = self.project.GetProjectLength() + self.EXTRA_SCROLL_TIME
		self.scrollRange.upper = length
		
		if self.centreViewOnPosition:  
			self.centreViewOnPosition = False  
			#set the view to be centred over the playhead  
			start = self.project.transport.GetPosition() - (self.scrollRange.page_size / 2)
			self.project.SetViewStart(start)
		# Need to adjust project view start if we are zooming out
		# and the end of the project is now before the end of the page.
		# Project end will be at right edge unless the start is also on 
		# screen, in which case the start will be at the left.
		elif self.project.viewStart + self.scrollRange.page_size > length:
			self.project.SetViewStart(length - self.scrollRange.page_size)
		
		#check the min zoom value (based on project length)
		# (scroll bar should always be same width as viewable area)
		pixelSize = self.scrollBar.allocation.width 
		minScale = pixelSize / length
		self.zoomSlider.set_range(minScale, self.ZOOM_MAX_SCALE)
		if self.zoomSlider.get_value() < minScale:
			self.zoomSlider.set_value(minScale)
		
	#_____________________________________________________________________
	
	def OnInstrumentAdded(self, project, instrument):
		"""
		Callback for when an instrument is added to the project.
		
		Parameters:
			project -- The project that the instrument was added to.
			instrument -- The instrument that was added.
		"""
		instrViewer = InstrumentViewer.InstrumentViewer(project, instrument, self, self.mainview, self.small)
		
		#Add it to the views
		self.views.append((instrument.id, instrViewer))
		self.header_size_group.add_widget(instrViewer.GetHeaderWidget())
		
		self.instrumentBox.pack_start(instrViewer, False, False)
		instrViewer.show()
	
	#_____________________________________________________________________
	
	def OnInstrumentRemoved(self, project, instrument):
		"""
		Callback for when an instrument is removed from the project.
		
		Parameters:
			project -- The project that the instrument was removed from.
			instrument -- The instrument that was removed.
		"""
		for ID, instrViewer in self.views:
			if ID == instrument.id:
				if instrViewer.parent:
					self.instrumentBox.remove(instrViewer)
				instrViewer.Destroy()
				self.views.remove((ID, instrViewer))
				break
	
	#_____________________________________________________________________
	
	def OnInstrumentReordered(self, project, instrument):
		"""
		Callback for when an instrument's position in the project has changed.
		
		Parameters:
			project -- The project that the instrument was changed on.
			instrument -- The instrument that was reordered.
		"""
		for ID, instrViewer in self.views:
			if ID == instrument.id:
				if instrViewer.parent:
					pos = self.project.instruments.index(instrument)
					self.instrumentBox.reorder_child(instrViewer, pos)
					instrViewer.show_all()
				break
		
	
	#_____________________________________________________________________
	
	def OnProjectGstError(self, project, error, debug):
		"""
		Callback for when the project sends a gstreamer error message
		from the pipeline.
		
		Parameters:
			project -- The project instance that send the signal.
			error -- The type of error that occurred as a string.
			debug -- A string with more debug information about the error.
		"""
		if not error:
			error = _("A Gstreamer error has occurred")
		
		if not self.errorMessageArea:
			msg_area = self.CreateDefaultErrorPane(error, debug)
			self.vbox.pack_end(msg_area, False, False)
			msg_area.show()
			self.errorMessageArea = msg_area
	
	#_____________________________________________________________________
	
	def CreateDefaultErrorPane(self, error, details):
		message = _("A GStreamer error has occurred.")
		info = _("If this problem persists consider reporting a bug using the link in the help menu.")
		
		details = "\n".join((error, info))
		
		msg_area = MessageArea.MessageArea()
		msg_area.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
		msg_area.set_text_and_icon(gtk.STOCK_DIALOG_ERROR, message, details)
		
		msg_area.connect("response", self.OnMessageAreaReponse, msg_area)
		msg_area.connect("close", self.OnMessageAreaClose, msg_area)
		
		return msg_area
		
	#_____________________________________________________________________
	
	def OnMessageAreaClose(self, widget, message_area):
		if self.errorMessageArea:
			self.vbox.remove(self.errorMessageArea)
			self.errorMessageArea = None
	
	#_____________________________________________________________________
	
	def OnMessageAreaReponse(self, widget, response_id, message_area):
		if response_id == gtk.RESPONSE_CLOSE:
			self.OnMessageAreaClose(widget, message_area)
	
	#_____________________________________________________________________
	
	def OnScroll(self, widget):
		"""
		Callback for "value-changed" signal from scrollbar.
		Updates the Project playhead position.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		pos = widget.get_value()
		self.project.SetViewStart(pos)

	#_____________________________________________________________________

	def OnZoom(self, widget):
		"""
		Updates the viewing scale for the Project when the user
		zooms in or out.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		# The left and right sides of the viewable area
		rightPos = self.project.viewStart + self.scrollRange.page_size
		leftPos = self.project.viewStart
		currentPos = self.project.transport.GetPosition()
		# Check if the playhead is currently viewable (don't force it in view if it isn't already in view)
		if leftPos < currentPos < rightPos:
			self.centreViewOnPosition = True
		
		#now do the zoom
		self.project.SetViewScale(widget.get_value())

	#_____________________________________________________________________
	
	def OnZoomOut(self, widget):
		"""
		Calls OnZoom when the user zooms out.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		tmp = self.project.viewScale * 4. / 5.
		#setting the value will trigger the "value-changed" signal and call OnZoom for us.
		self.zoomSlider.set_value(tmp)
		
	#_____________________________________________________________________
		
	def OnZoom100(self, widget):
		"""
		This method is not currently used (it was used when the zoom buttons existed).
		It's left here for future use.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		self.project.SetViewScale(25.0)
		
	#_____________________________________________________________________
		
	def OnZoomIn(self, widget):
		"""
		Calls OnZoom when the user zooms in.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		tmp = self.project.viewScale * 1.25
		#setting the value will trigger the "value-changed" signal and call OnZoom for us.
		self.zoomSlider.set_value(tmp)

	#_____________________________________________________________________
		
	def OnZoomReset(self, widget, mouse):
		"""
		Calls OnZoom when the user resets the zoom to the default by right-clicking.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if mouse.button == 3:
			tmp = (self.ZOOM_MAX_SCALE - self.ZOOM_MIN_SCALE) / 2
			#setting the value will trigger the "value-changed" signal and call OnZoom for us.
			self.zoomSlider.set_value(tmp)
			return True
		
	#_____________________________________________________________________
	
	def OnViewStartChanged(self, project):
		"""
		Callback for when the project notifies that the
		viewable start position has changed.
		
		Parameters:
			project -- The project instance that send the signal.
		"""
		self.scrollRange.value = project.viewStart
	
	#_____________________________________________________________________
	
	def OnMouseDown(self, widget, mouse):
		"""
		Callback for "button_press_event" (not catered for, by any
		button presses or other mouse handlers).
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		# If we're here then we're out of bounds of anything else
		# So we should clear any selected events
		if mouse.type == gtk.gdk._2BUTTON_PRESS or mouse.type == gtk.gdk._3BUTTON_PRESS:
			return True
		self.project.ClearEventSelections()
		self.project.SelectInstrument(None)
		
	#_____________________________________________________________________
	
	def OnEmptySpaceDoubleClicked(self, widget, mouse):
		"""
		Callback for "button_press_event" (not catered for, by any
		button presses or other mouse handlers).
		Shows the add instrument dialog when the empty space is double clicked.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
			mouse -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if mouse.type == gtk.gdk._2BUTTON_PRESS:
			self.mainview.OnShowAddInstrumentDialog()
			
	#_____________________________________________________________________
	
	def OnDragDataReceived(self, widget, context, x, y, selection, targetType, time):
		"""
		Called when the user releases MOUSE1, finishing a drag and drop
		procedure.
		Adds an instrument and event for each "file://"-uri in the uri-list to the Instrument, 
		one after the other. The files will be copied to the Project's audio directory.
			
		Parameters:
			widget -- InstrumentViewer being dragged.
			context -- reserved for GTK callbacks, don't use it explicitly.
			x -- point in the X axis the dragged object was dropped.
			y -- point in the Y axis the dragged object was dropped..
			selection -- selected object area that was dragged.
			targetType -- mimetype of the dragged object.
			time -- reserved for GTK callbacks, don't use it explicitly.
			
		Returns:
			True -- continue GTK signal propagation. *CHECK*
		"""
		# Splitlines to separate the uri's, unquote to decode the uri-encoding ('%20' -> ' ')
		uris = [urllib.unquote(uri) for uri in selection.data.splitlines()]
		self.project.AddInstrumentAndEvents(uris, True) # True: copy
		
		context.finish(True, False, time)
		return True
	
	#_____________________________________________________________________
	
	def OnDragMotion(self, widget, context, x, y, time):
		"""
		Called each time the user moves the mouse onto this widget while dragging.
		
		Parameters:
			widget -- InstrumentViewer the mouse is hovering over.
			context -- cairo widget context.
			x -- reserved for GTK callbacks, don't use it explicitly.
			y -- reserved for GTK callbacks, don't use it explicitly.
			time -- reserved for GTK callbacks, don't use it explicitly.
		
		Returns:
			True -- continue GTK signal propagation. *CHECK*
		"""
		context.drag_status(gtk.gdk.ACTION_COPY, time)
		return True
	
	#_____________________________________________________________________
	
	def ChangeSize(self, small):
		"""
		Alters the size of the instrument lanes and removes the zoom buttons.
		
		Parameters:
			small -- True if changing to small. Otherwise False.
		"""
		#if the requested size has not changed then quit
		if small == self.small:
			return
		self.small = small
		children = self.instrumentBox.get_children()
		for instrView in children:
			instrView.ChangeSize(small)
		if self.small:
			self.inbutton.hide()
			self.outbutton.hide()
			self.zoomSlider.hide()
		else:
			self.inbutton.show()
			self.outbutton.show()
			self.zoomSlider.show()
			
	#____________________________________________________________________	
	
	def OnRestoreMessageAreaClose(self, widget=None, msg_area=None):
		if self.restoreMessageArea:
			self.vbox.remove(self.restoreMessageArea)
			self.restoreMessageArea = None
	
	#____________________________________________________________________
	
	def OnRestoreMessageAreaResponse(self, widget, response_id, msg_area):
		if response_id == gtk.RESPONSE_OK:
			self.project.DoIncrementalRestore()
		
		self.OnRestoreMessageAreaClose()
	
	#____________________________________________________________________
	
	def OnProjectIncSave(self, project):
		self.OnRestoreMessageAreaClose()
	
	#____________________________________________________________________
#=========================================================================



#
#       THIS FILE IS PART OF THE JOKOSHER PROJECT AND LICENSED UNDER THE GPL. SEE
#       THE 'COPYING' FILE FOR DETAILS
#
#       MixerStrip.py
#       
#		Contains the VU widget, the buttons below it and levels,
#		for an Instrument in the mixing view.
#
#-------------------------------------------------------------------------------

import pygtk
pygtk.require("2.0")
import gtk
import gobject
import os
import Globals
import Utils
import ControlsBox

import gettext
_ = gettext.gettext

from VUWidget import *

#=========================================================================

class MixerStrip(gtk.Frame):
	"""
	Contains the VU widget and the buttons below it,
	for an Instrument in the mixing view.
	"""
	
	""" GTK widget name """
	__gtype_name__ = 'MixerStrip'
	
	#_____________________________________________________________________
	
	def __init__(self, project, instrument, mixview, mainview):
		"""
		Creates a new instance of MixerStrip.
		
		Parameters:
			project -- the currently active Project.
			instrument -- the instrument associated with this MixerStrip.
			mixview -- the mixing view object (CompactMixView).
			mainview -- the main Jokosher window (MainApp).
		"""
		gtk.Container.__init__(self)
		self.project = project
		self.instrument = instrument
		self.mixview = mixview
		self.mainview = mainview
		self.Updating = False
		self.statusbarMsgID = None
		
		self.vbox = gtk.VBox()
		self.add(self.vbox)

		self.minbutt = gtk.Button()
		img = gtk.image_new_from_stock(gtk.STOCK_GOTO_BOTTOM, gtk.ICON_SIZE_MENU)
		self.minbutt.set_image(img)
		self.mintip = gtk.Tooltips()
		self.mintip.set_tip(self.minbutt, _("Minimize instrument"), None)
		self.minbutt.connect("clicked", self.EmitMinimise)
				
		self.vbox.pack_start(self.minbutt, False)
		
		self.panvbox = gtk.VBox()
		self.panvbox.set_border_width(3)
		self.vbox.pack_start(self.panvbox, False)

		# the slider label
		balanceLabel = gtk.Label(_("Balance:"))
		self.panvbox.pack_start(balanceLabel, False)
		# add the panning slider
		self.panhbox = gtk.HBox()
		self.panhbox.set_spacing(3)
		self.leftlab = gtk.Label(_("L"))
		self.rightlab = gtk.Label(_("R"))
		self.pan = gtk.HScale()
		self.pan.set_range(-1.0, 1.0)
		self.pan.set_increments(0.1, 1.0)
		self.pan.set_draw_value(False)
		self.pantip = gtk.Tooltips()
		self.pantip.set_tip(self.pan, _("Adjust instrument balance. Right-click to center"), None)
		
		if self.instrument.pan:
			self.pan.set_value(self.instrument.pan)
		
		self.pan.connect("value-changed", self.OnPanChanged)
		self.pan.connect("button-press-event", self.OnPanClicked)
		self.panhbox.pack_start(self.leftlab, False)
		self.panhbox.pack_start(self.pan, True)
		self.panhbox.pack_start(self.rightlab, False)
		
		self.panvbox.pack_start(self.panhbox, False)

		#volume label
		volumeLabel = gtk.Label(_("Volume:"))
		volumeLabel.set_padding(3, 3)
		self.vbox.pack_start(volumeLabel, False)
		# VU Meter
		self.vu = VUWidget(self, self.mainview)
		self.vbox.pack_start(self.vu, True, True)
		
		#Control Buttons
		controlsBox = ControlsBox.ControlsBox(project, mainview,instrument,includeEffects=False)
		self.vbox.pack_start(controlsBox, False, False)
		
		# Label and icon
		hb = gtk.HBox()
		self.instrImage = gtk.Image()
		#initalize the image from the instrument's pixbuf
		self.OnInstrumentImage()
		hb.pack_start(self.instrImage, False, False)
		
		self.label = gtk.Label(instrument.name)
		self.label.set_max_width_chars(6)
		hb.pack_start(self.label, True, True)

		self.instrument.connect("name", self.OnInstrumentName)
		self.instrument.connect("image", self.OnInstrumentImage)
		
		self.vbox.pack_end(hb, False, False)
		self.vbox.show_all()
		self.show_all()
		
	#_____________________________________________________________________

	def EmitMinimise(self, widget):
		"""
		Minimizes the Instrument to the StatusBar.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		self.emit("minimise")
	
	#_____________________________________________________________________
	
	def OnInstrumentName(self, instrument=None):
		"""
		Callback for when the instrument's name changes.
		
		Parameters:
			instrument -- the instrument instance that send the signal.
		"""
		self.label.set_text(self.instrument.name)
		
	#_____________________________________________________________________
	
	def OnInstrumentImage(self, instrument=None):
		"""
		Callback for when the instrument's image changes.
		
		Parameters:
			instrument -- the instrument instance that send the signal.
		"""
		imgsize = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)[0]
		pixbuf = self.instrument.pixbuf.scale_simple(imgsize, imgsize, gtk.gdk.INTERP_BILINEAR)
		self.instrImage.set_from_pixbuf(pixbuf)
		
	#_____________________________________________________________________

	def Destroy(self):
		"""
		Called when the MixerStrip is destroyed. It also emits the
		destroy signal to the VUMixer widget.
		"""
		self.instrument.disconnect_by_func(self.OnInstrumentImage)
		self.instrument.disconnect_by_func(self.OnInstrumentName)
		self.vu.Destroy()
		self.destroy()
	
	#_____________________________________________________________________

	def GetLevel(self):
		"""
		Obtain the instrument level.
		
		Returns:
			the level of the instrument.
		"""
		return self.instrument.level
		
	#_____________________________________________________________________

	def GetVolume(self):
		"""
		Obtain the instrument volume.
		
		Returns:
			the volume of the instrument.
		"""
		return self.instrument.volume
		
	#_____________________________________________________________________

	def SetVolume(self, vol):
		"""
		Sets the instrument's volume.
		
		Parameters:
			vol -- volume value to set this Instrument's volume to.
		"""
		self.instrument.SetVolume(vol)
		
	#_____________________________________________________________________
	
	def CommitVolume(self):
		self.instrument.CommitVolume()
	
	#_____________________________________________________________________
	
	def OnPanChanged(self, slider):
		"""
		Changes the Instrument's audiopanorama value to the
		one indicated by the slider control.
		It also updates the statusbar when dragging the slider.
		
		Parameters:
			slider -- panning slider control.
		"""
		value = round(slider.get_value(), 2)
		
		# clear any existing status bar messages
		if self.statusbarMsgID is not None:
			self.mainview.ClearStatusBar(self.statusbarMsgID)
		
		# set the statusbar message depending on the current pan value
		if value < 0:
			self.statusbarMsgID = self.mainview.SetStatusBar(_("Current balance is <b>%d%%</b> left") % (-value * 100))
		elif value == 0:
			self.statusbarMsgID = self.mainview.SetStatusBar(_("Current balance is <b>centered</b>"))
		else:
			self.statusbarMsgID = self.mainview.SetStatusBar(_("Current balance is <b>%d%%</b> right") % (value * 100))
		
		#to remove the status bar message in a few seconds
		self.OnTimedStatusBarClear()
		
		self.instrument.pan = value
		self.instrument.panElement.set_property("panorama", value)
		
	#_____________________________________________________________________
	
	def OnPanClicked(self, slider, mouse):
		"""
		Centers the panning slider bar when right clicked.
		
		Parameters:
			slider -- panning slider control.
			mouse -- mouse event that fired this callback.
		"""
		if mouse.button == 3:
			slider.set_value(0.0)
			return True
			
	#_____________________________________________________________________
	
	def OnTimedStatusBarClear(self):
		"""
		Waits for a few seconds and then clears the status bar message.
		"""
		# clear any existing status bar messages
		if self.statusbarMsgID is not None:
			gobject.timeout_add(2000, self.mainview.ClearStatusBar, self.statusbarMsgID)
			self.statusbarMsgID = None
	
	#_____________________________________________________________________
#=========================================================================

#create signal to be emitted by MixerStrip
gobject.signal_new("minimise", MixerStrip, gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ())

#=========================================================================



#
#	THIS FILE IS PART OF THE JOKOSHER PROJECT AND LICENSED UNDER THE GPL. SEE
#	THE 'COPYING' FILE FOR DETAILS
#
#	ProjectTemplateDialog.py
#	
#	This module is used to present a dialog which allows the user to create, modify and
#	remove project templates.
#
#-------------------------------------------------------------------------------

import gtk
import gtk.glade
import Globals
import ProjectTemplate
import gettext
_ = gettext.gettext

#=========================================================================

class ProjectTemplateDialog:
	"""
	This class allows the user to modify, save and delete project templates.
	"""
	
	#_____________________________________________________________________
	
	def __init__(self, newprojectdlg, template):
		""" 
		Creates a new instance of ProjectTemplateDialog.
		
		Parameters:
			newprojectdlg -- reference to NewProjectDialog.
			template -- reference to ProjectTemplate.
		"""
		self.newprojectdlg = newprojectdlg
		self.template = template

		self.wTree = gtk.glade.XML(Globals.GLADE_PATH, "ProjectTemplateDialog")
		

		signals = {
			"on_save_button_clicked" : self.OnSaveButtonClicked,
			"on_delete_button_clicked" : self.OnDeleteButtonClicked,
			"on_add_button_clicked" : self.OnAddButtonClicked,
			"on_remove_button_clicked": self.OnRemoveButtonClicked,
			"on_close_button_clicked" : self.OnCloseButtonClicked,
		}
		self.wTree.signal_autoconnect(signals)
		
		self.window = self.wTree.get_widget("ProjectTemplateDialog")
		self.window.set_transient_for(self.newprojectdlg.dlg)
		self.window.set_icon(self.newprojectdlg.dlg.get_icon())
		
		self.tempcombo = self.wTree.get_widget("template_comboentry")
		self.tempcombo.child.set_activates_default(True)
		
		self.instrtree = self.wTree.get_widget("instrument_treeview")
		self.addinstrbtn = self.wTree.get_widget("add_button")
		self.removetempinstr = self.wTree.get_widget("remove_button")
		self.temptree = self.wTree.get_widget("template_treeview")
		
		self.savebtn = self.wTree.get_widget("save_button")
		self.savebtn.set_flags(gtk.CAN_DEFAULT)
		self.savebtn.grab_default()
		self.deletebtn = self.wTree.get_widget("delete_button")
		
		self.temptreemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
		self.instrtreemodel = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
		
		# the template combo entry uses the same model as the template combo in new project dialog
		self.tempcombo.set_model(self.newprojectdlg.templatemodel)
		self.instrtree.set_model(self.instrtreemodel)		
		self.temptree.set_model(self.temptreemodel)
		
		self.tempcombo.clear()
		textrend = gtk.CellRendererText()
		self.tempcombo.pack_start(textrend)
		self.tempcombo.add_attribute(textrend, "text", 0)
		
		self.instrtree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.instrtree.append_column(gtk.TreeViewColumn(_("Type"), gtk.CellRendererPixbuf(), pixbuf=0))
		self.instrtree.append_column(gtk.TreeViewColumn(_("Instrument Name"), gtk.CellRendererText(), text=2))
		
		self.temptree.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.temptree.append_column(gtk.TreeViewColumn(_("Type"), gtk.CellRendererPixbuf(), pixbuf=0))
		self.temptree.append_column(gtk.TreeViewColumn(_("Instrument Name"), gtk.CellRendererText(), text=2))
		
		self.tempcombo.connect("changed", self.OnTemplateEntryChanged)

		self.PopulateJokosherInstrumentsModel()
		self.tempcombo.set_active(self.newprojectdlg.templatecombo.get_active())
		self.template.connect("template-update", self.OnTemplateUpdate)
		self.window.show_all()
		
	#_____________________________________________________________________		
		
	def OnTemplateEntryChanged(self, widget):
		"""
		Called when the template combo entry contents change.
			
		Parameters:
			widget -- reserved for GTK callbacks. Don't use explicitly.
		"""
		if self.tempcombo.child.get_text():
			self.savebtn.set_sensitive(True)
			self.deletebtn.set_sensitive(True)
			self.Update()
		else:
			self.savebtn.set_sensitive(False)
			self.deletebtn.set_sensitive(False)
			
	#_____________________________________________________________________
	
	def Update(self):
		"""
		Called when the template combo entry contents change.
		Loads instruments from a template file and appends them to the template instruments model.
		"""
		for key, value in self.template.LoadDictionaryOfInstrumentsFromTemplateFile().iteritems():
			if key == self.tempcombo.child.get_text():
				self.temptreemodel.clear()
				for name, type, pixbuf in value:
					pixbuf = pixbuf.scale_simple(22, 22, gtk.gdk.INTERP_BILINEAR)
					self.temptreemodel.append( (pixbuf, type, name) )
				break
	
	#_____________________________________________________________________
	
	def OnAddButtonClicked(self, widget):
		"""
		Called when the add button is clicked.
		Adds selected instruments to the template instruments model.
		
		Parameters:
			widget -- reserved for GTK callbacks. Don't use explicitly.
		"""
		selection = self.instrtree.get_selection()
		if selection:
			model, selected = selection.get_selected_rows()
			iters = [model.get_iter(path) for path in selected]
			for iter in iters:
				self.temptreemodel.append( (model[iter][0], model[iter][1], model[iter][2]) )
		else:
			return

	#_____________________________________________________________________
	
	def OnRemoveButtonClicked(self, widget):
		"""
		Called when the remove button is clicked.
		Removes any selected instruments from the template instruments model.
		
		Parameters:
			widget -- reserved for GTK callbacks. Don't use explicitly.
		"""
		selection = self.temptree.get_selection()
		if selection:
			model, selected = selection.get_selected_rows()
			iters = [model.get_iter(path) for path in selected]
			for iter in iters:
				model.remove(iter)
		else:
			return

	#_____________________________________________________________________
	
	def OnSaveButtonClicked(self, widget):
		"""
		Called when the save button is clicked.
		Saves added template instruments to a template file.
		
		Parameters:
			widget -- reserved for GTK callbacks. Don't use explicitly.
		"""
		instruments = []
		for row in self.temptreemodel:
			instruments.append(row[1])
		self.template.SaveTemplateFile(self.tempcombo.child.get_text(), instruments)
		
	#_____________________________________________________________________
			
	def OnDeleteButtonClicked(self, widget):
		"""
		Called when the delete button is clicked.
		Removes the selected template file from disk.
		
		Parameters:
			widget -- reserved for GTK callbacks. Don't use explicitly.
		"""
		self.template.DeleteTemplateFile(self.tempcombo.child.get_text())

	#_____________________________________________________________________
		
	def OnCloseButtonClicked(self, widget):
		"""
		Called when the close button is clicked.
		Destroys the project templates window.
		
		Parameters:
			widget -- reserved for GTK callbacks. Don't use explicitly.
		"""
		self.window.destroy()
		
	#_____________________________________________________________________
	
	def OnTemplateUpdate(self, projectTemplate):
		"""
		Callback for when the ProjectTemplate instance for this dialog
		sends a signal to update.
		
		Parameters:
			projectTemplate -- The ProjectTemplate instance that send the signal.
		"""
		self.UpdateTemplateModel()
	
	#_____________________________________________________________________
	
	def PopulateJokosherInstrumentsModel(self):
		"""
		Called when the jokosher instruments model needs to be populated.
		"""
		instrlist = [(x[0], x[1], x[2]) for x in Globals.getCachedInstruments()]
		for name, type, pixbuf in instrlist:
			pixbuf = pixbuf.scale_simple(22, 22, gtk.gdk.INTERP_BILINEAR)
			self.instrtreemodel.append( (pixbuf, type, name) )	
		
	#_____________________________________________________________________
	
	def UpdateTemplateModel(self):
		"""
		Called when the template model needs updating.
		"""
		self.newprojectdlg.templatemodel.clear()
		for item in self.template.LoadDictionaryOfInstrumentsFromTemplateFile().iterkeys():
			self.newprojectdlg.templatemodel.append((item,))
			self.newprojectdlg.templatecombo.show_all()
			self.newprojectdlg.templatecombo.set_active(0)
		
	#_____________________________________________________________________
		
#=========================================================================



#
#	THIS FILE IS PART OF THE JOKOSHER PROJECT AND LICENSED UNDER THE GPL. SEE
#	THE 'COPYING' FILE FOR DETAILS
#
#	This module is used to present a dialog which allows the user to create, modify and
#	remove mixdown profiles.
#
#-------------------------------------------------------------------------------

import gtk.glade
import gobject

import Globals
import MixdownProfileManager
import MixdownActions

import gettext
_ = gettext.gettext

#=========================================================================

class MixdownProfileDialog:
	"""
	This class allows the user to create, modify and remove mixdown profiles.
	"""
	
	#_____________________________________________________________________

	def __init__(self, mainapp, project, profile=None):
		"""
		Creates a new instance of MixdownProfileDialog.
		
		Parameters:
			project -- the currently active Project.
			mainapp -- reference to the MainApp Jokosher window.
		"""
		if profile:
			self.profile = profile
		else:
			self.profile = None
			
		self.mainapp = mainapp
		self.project = project
		
		self.res = gtk.glade.XML(Globals.GLADE_PATH, "MixdownProfileDialog")

		self.signals = {
			"on_add_profile_button_clicked" : self.OnAddProfile,
			"on_remove_profile_button_clicked" : self.OnRemoveProfile,
			"on_configure_action_button_clicked" : self.OnConfigureAction,
			"on_add_action_button_clicked" : self.OnAddAction,
			"on_remove_action_button_clicked" : self.OnRemoveAction,
			"on_cancel_button_clicked" : self.OnDestroy,
			"on_mixdown_button_clicked" : self.OnMixdown
		}

		self.res.signal_autoconnect(self.signals)

		self.window = self.res.get_widget("MixdownProfileDialog")
		self.window.set_default_size(450, 400)
		self.window.set_icon(self.mainapp.icon)

		self.profileCombo = self.res.get_widget("profile_combo")
		self.treeView = self.res.get_widget("actions_treeview")
		self.mixdownButton = self.res.get_widget("mixdown_button")
		self.configureLabel = self.res.get_widget("action_configured_label")
		
		self.manager = MixdownProfileManager.MixdownProfileManager(self)
		
		self.treeViewModel = gtk.ListStore(gtk.gdk.Pixbuf, str, object) # pixbuf, details, class instance
		self.profileComboModel = gtk.ListStore(str)
		
		self.treeView.set_model(self.treeViewModel)
		self.profileCombo.set_model(self.profileComboModel)
		
		iconCell = gtk.CellRendererPixbuf()
		iconColumn = gtk.TreeViewColumn()
		iconColumn.pack_start(iconCell, False)
		iconColumn.add_attribute(iconCell, "pixbuf", 0)
		
		detailsCell = gtk.CellRendererText()
		detailsColumn = gtk.TreeViewColumn()
		detailsColumn.pack_start(detailsCell, False)
		detailsColumn.add_attribute(detailsCell, "markup", 1)
		
		self.treeView.append_column(iconColumn)
		self.treeView.append_column(detailsColumn)
		
		self.profileCombo.clear()
		textrend = gtk.CellRendererText()
		self.profileCombo.pack_start(textrend)
		self.profileCombo.add_attribute(textrend, "text", 0)
		self.profileCombo.connect("changed", self.OnProfileComboBoxChanged)
		
		self.treeViewSelection = self.treeView.get_selection()
		self.treeViewSelection.connect("changed", self.OnSelectionChanged)
		self.treeViewSelection.set_mode(gtk.SELECTION_SINGLE)
		
		self.PopulateProfileComboBoxModel()
		
		# the previously selected item in the treeview
		self.lastSelected = None
		# the previously selected action instance in the treeview
		self.lastAction = None
		
		if self.profile:
			self.SetActiveProfileItem(self.profile)
		elif self.CountRowsInTreeModel(self.profileComboModel) > 0:
			self.profileCombo.set_active(0)
			
		self.window.show_all()
		
	#_____________________________________________________________________
	
	def SetActiveProfileItem(self, profileName):
		"""
		Called when an item in the profile combo (self.profileCombo) should be made
		the active item.
		Sets the active item in profile combo by the profile name specified
		
		Parameters:
			profileName -- name of the profile which should be active.
		"""
		active = -1
		for item in self.profileComboModel:
			active += 1
			if item[0] == self.profile:
				self.profileCombo.set_active(active)
				break
			
	#_____________________________________________________________________
	
	def OnSelectionChanged(self, widget):
		"""
		Called when a treeview (self.treeView) selection has been made.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.treeViewSelection.count_selected_rows() > 0:
			self.OnCheckActionConfigured()
			self.UpdateSelectedActionAppearence()
		else:
			return
		
	#_____________________________________________________________________
	
	def UpdateSelectedActionAppearence(self):
		"""
		Called when a MixdownAction is selected in the action treeview (self.treeView).
		Changes the appearence of the selected action.
		"""
		selected = self.treeViewSelection.get_selected()
		action = self.treeViewModel [selected[1]] [2]
		
		# set the foreground colour of the text to white
		newText = "<span size='larger' weight='bold'>%s</span>\n" % action.name  \
		+ "<span size='smaller' foreground='white'>%s</span>" % action.description
		self.treeViewModel [selected[1]] [1] = newText
		
		# modify the last selected item in the model to use the default colour grey
		if self.lastSelected:
			oldText = "<span size='larger' weight='bold'>%s</span>\n" % self.lastAction.name  \
			+ "<span size='smaller' foreground='dim grey'>%s</span>" % self.lastAction.description
			self.treeViewModel [self.lastSelected[1]] [1] = oldText
		
		self.lastAction = action
		self.lastSelected = selected
		
	#_____________________________________________________________________
	
	def OnCheckActionConfigured(self):
		"""
		Called to check if a MixdownAction has been configured.
		Checks to see if the selected action is configured and updates
		the configure label accordingly.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.treeViewSelection.count_selected_rows() > 0:
			profileName = self.profileComboModel[self.profileCombo.get_active()][0]
			selected = self.treeViewSelection.get_selected()
			action = self.treeViewModel [selected[1]] [2]
			if action.isConfigured:
				self.configureLabel.set_markup(_("<b>%s is configured</b>" % action.name))
			else:
				self.configureLabel.set_markup(_("<b>%s is not configured</b>" % action.name))
		else:
			self.configureLabel.set_text("")
			
	#_____________________________________________________________________
	
	def AddActionToActionModel(self, profileName, action):
		"""
		Called when a MixdownAction needs to be added to the action model (self.treeViewModel).
		Adds the information associated with the MixdownAction to the action model.
		
		Parameters:
			profileName -- name of the mixdown profile to add the action to.
			action -- the MixdownAction instance which will be added to the action model.
		"""
		self.treeViewModel.append( self.ReturnActionDisplayDetails(action) )
		self.SaveProfileActions(profileName)
		
	#_____________________________________________________________________
	
	def ReturnActionDisplayDetails(self, action):
		"""
		Called when action details need to be displayed in the action model (self.treeViewModel).
		Returns the details needed for the action to be added to the treeview (self.treeView).
		
		Parameters:
			action -- the MixdownAction instance which will be added to the action model.
		"""
		pixbuf = self.ReturnActionPixbuf(action)
		
		detailsText = "<span size='larger' weight='bold'>%s</span>\n" % action.name  \
		+ "<span size='smaller' foreground='dim grey'>%s</span>" % action.description
		
		return (pixbuf, detailsText, action)
		
	#_____________________________________________________________________
	
	def UpdateProfileModel(self, signalDetails):
		"""
		Called when the profile combo model (self.profileComboModel) needs updating.
		Updates the combo box model with the files that reside in the mixdownprofiles directory
		(JOKOSHER_DATA_HOME/mixdownprofiles/).
		
		Parameters:
			signalName -- the signal details which are passed to this method.
		"""
		profileName = None
		active = self.profileCombo.get_active()

		if signalDetails == "deleteProfile":
			if active:
				active -= 1
		
		self.profileComboModel.clear()
		for item in self.manager.GetMixdownProfileList():
			self.profileComboModel.append((item,))
			self.profileCombo.show_all()
			if active > 0:
				self.profileCombo.set_active(active)
			else:
				self.profileCombo.set_active(0)
		
		if self.CountRowsInTreeModel(self.profileComboModel) > 0:
			profileName = self.profileComboModel[self.profileCombo.get_active()][0]
		self.UpdateActionTreeViewModel(profileName)
			
	#_____________________________________________________________________
		
	def UpdateActionTreeViewModel(self, profileName):
		"""
		Called when the action treeview (self.treeView) needs updating.
		Updates the MixdownActions in the treeview.
		
		Parameters:
			profileName -- the name of the profile to use to retrieve MixdownActions from.
		"""
		self.treeViewModel.clear()
		if profileName:
			actions = self.manager.ReturnAllActionsFromMixdownProfile(profileName)
			if actions:
				for action in actions:
					action.connect("action-configured", self.ActionIsConfigured)
					self.treeViewModel.append( self.ReturnActionDisplayDetails(action) )
		self.OnCheckActionConfigured()

	#_____________________________________________________________________
	
	def ActionIsConfigured(self, action):
		"""
		Called when a MixdownAction has been configured.
		Saves the MixdownActions present in the action treeview (self.treeView)
		to the currently selected profile (self.profileCombo).
		
		Parameters:
			action -- the MixdownAction instance which has just been configured.
		"""
		profileName = self.profileComboModel[self.profileCombo.get_active()][0]
		self.SaveProfileActions(profileName)
		
	#_____________________________________________________________________
	
	def ReturnActionPixbuf(self, action):
		"""
		Called when the action treeview (self.treeView) needs to insert a
		pixbuf associated with a MixdownAction.
		Returns a pixbuf from the icon path specified by the MixdownAction.
		
		Parameters:
			action -- the MixdownAction instance from which a pixbuf will be returned.
			
		Returns:
			pixbuf -- A gtk.gdk.Pixbuf associated with the MixdownAction.
		"""
		if action.iconPath.startswith("gtk"):
			pixbuf = self.treeView.render_icon(action.iconPath, gtk.ICON_SIZE_DIALOG)
		else:
			pixbuf = gtk.gdk.pixbuf_new_from_file(action.iconPath)
	
		if pixbuf.get_property("width") and pixbuf.get_property("height") != 48:
			scaled = pixbuf.scale_simple(48, 48, gtk.gdk.INTERP_BILINEAR)
			return scaled
		else:
			return pixbuf
	
	#_____________________________________________________________________

	def PopulateProfileComboBoxModel(self):
		"""
		Populates the profile combo model (self.profileComboModel) with profiles
		in JOKOSHER_DATA_HOME/mixdownprofiles/
		"""
		for profile in self.manager.GetMixdownProfileList():
			self.profileComboModel.append((profile,))
			
	#_____________________________________________________________________
	
	def OnProfileComboBoxChanged(self, widget):
		"""
		Called when the profile combo box (self.profileCombo) contents change.
		Updates the action treeview (self.treeView) with the MixdownActions in
		the newly selected profile.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.CountRowsInTreeModel(self.profileComboModel) > 0:
			profileName = self.profileComboModel[self.profileCombo.get_active()][0]
			self.UpdateActionTreeViewModel(profileName)
		else:
			# if there is nothing in the combo model, then clear any actions that may be
			# remaining in the action treeview
			self.treeViewModel.clear()
			
	#_____________________________________________________________________

	def OnAddProfile(self, widget):
		"""
		Called when the Add Mixdown Profile button is clicked.
		Shows a dialog allowing the user to add MixdownActions
		to the currently selected profile (self.profileCombo).
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		self.ShowAddProfileDialog()
		
	#_____________________________________________________________________
	
	def ShowAddProfileDialog(self):
		"""
		Shows the Add Mixdown Profile dialog, allowing the user to create a MixdownProfile.
		"""
		buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK)
		
		iconBox = gtk.HBox()
		iconBox.set_border_width(12)
		iconBox.set_spacing(12)
		iconImage = gtk.Image()
		iconImage.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
		iconLabel = gtk.Label(_("Please enter the name of the mixdown profile you wish to create."))
		iconLabel.set_line_wrap(True)
		
		iconBox.pack_start(iconImage, False, False)
		iconBox.pack_start(iconLabel, False, False)
		
		entryBox = gtk.HBox()
		entryBox.set_spacing(12)
		entryBox.set_border_width(6)
		profileEntry = gtk.Entry()
		entryBox.pack_start(gtk.Label(_("Profile name:")), False, False)
		entryBox.pack_start(profileEntry, True, True)
		
		dlg = gtk.Dialog(_("Create Mixdown Profile"), self.window, gtk.DIALOG_DESTROY_WITH_PARENT, buttons)
		dlg.set_default_size(375, 200)
		dlg.set_has_separator(False)
		dlg.set_default_response(gtk.RESPONSE_OK)
		
		dlg.vbox.set_spacing(6)
		dlg.vbox.pack_start(iconBox, False, False)
		dlg.vbox.pack_start(entryBox, False, False)
		dlg.vbox.show_all()
		response = dlg.run()
		
		if response == gtk.RESPONSE_OK:
			msgdlg = gtk.MessageDialog(self.window,
					gtk.DIALOG_DESTROY_WITH_PARENT,
					gtk.MESSAGE_INFO,
					gtk.BUTTONS_CLOSE)
			if profileEntry.get_text():
				self.manager.SaveMixdownProfile(profileEntry.get_text())
				msgdlg.set_markup(_("Successfully created mixdown profile <b>%s.profile</b>.") % profileEntry.get_text())
				msgdlg.run()
				msgdlg.destroy()
			else:
				msgdlg.set_markup(_("Cannot create mixdown profile. Please make sure you have specified a profile name."))
				msgdlg.set_property("message-type", gtk.MESSAGE_ERROR)
				msgdlg.run()
				msgdlg.destroy()
		dlg.destroy()
		
	#_____________________________________________________________________
	
	def ShowActionErrorDialog(self, actionName, extensionName):
		"""
		Called when an error has occured while loading MixdownActions.
		Shows a dialog informing the user that a MixdownAction has failed to load.
		
		Parameters:
			actionName -- the name of the MixdownAction which cannot be loaded.
			extensionName -- the name of the extension that the MixdownAction can't be loaded from.
		"""
		msgdlg = gtk.MessageDialog(self.window,
				gtk.DIALOG_DESTROY_WITH_PARENT,
				gtk.MESSAGE_ERROR,
				gtk.BUTTONS_CLOSE)
		message = _("Please make sure the extension <b>%s</b> is enabled." % extensionName)
		msgdlg.set_markup(_("<big><b>Cannot load mixdown action %s.</b></big>\n\n%s" % (actionName, message)))
		msgdlg.run()
		msgdlg.destroy()
		
	#_____________________________________________________________________

	def OnRemoveProfile(self, widget):
		"""
		Called when the Remove Mixdown Profile button is clicked.
		Removes the currently selected profile in the profile
		combo box (self.profileCombo).
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.CountRowsInTreeModel(self.profileComboModel) > 0:
			profileName = self.profileComboModel[self.profileCombo.get_active()][0]
			self.manager.DeleteMixdownProfile(profileName)
		else:
			return
	
	#_____________________________________________________________________

	def OnConfigureAction(self, widget):
		"""
		Called when the Configure Mixdown Action button is clicked.
		Calls the selected MixdownAction's ConfigureAction method.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.treeViewSelection.count_selected_rows() > 0:
			selected = self.treeViewSelection.get_selected()
			action = self.treeView.get_model() [selected[1]] [2]
			action.ConfigureAction()
		else:
			return
		
	#_____________________________________________________________________

	def OnAddAction(self, widget):
		"""
		Called when the Add Mixdown Action button is clicked.
		Shows the Add Mixdown Action dialog, allowing the user to add a MixdownAction
		to the currently selected profile (self.profileCombo).
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.CountRowsInTreeModel(self.profileComboModel) > 0:
			if self.profileComboModel[self.profileCombo.get_active()][0]:
				self.ShowAddActionDialog()
		else:
			return

	#_____________________________________________________________________

	def OnRemoveAction(self, widget):
		"""
		Called when the Remove Action button is clicked.
		Removes the currently selected action from the action
		treeview (self.treeView)
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.treeViewSelection.count_selected_rows() > 0:
			profileName = self.profileComboModel[self.profileCombo.get_active()][0]
			iterpos = self.treeViewSelection.get_selected()[1]
			self.treeView.get_model().remove(iterpos)
			self.SaveProfileActions(profileName)
		else:
			return
	
	#_____________________________________________________________________
	
	def ShowAddActionDialog(self):
		"""
		Called when the Add Mixdown Action button is clicked.
		Shows a dialog which allows the user to add a MixdownAction
		to the currently selected profile (self.profileCombo)
		"""
		AddMixdownActionDialog(self)
	
	#_____________________________________________________________________

	def SaveProfileActions(self, name):
		"""
		Called when MixdownActions in a profile need to be saved.
		Saves MixdownActions to the profile specified.
		
		Parameters:
			name -- name of the profile which the mixdown actions will be saved to.
		"""
		actions = []
		for row in self.treeView.get_model():
			actions.append(row[2])
		self.manager.SaveMixdownProfile(name, actions)
		
	#_____________________________________________________________________

	def OnDestroy(self, widget):
		"""
		Called when the window is closed. Destroys the window.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		self.window.destroy()
	
	#_____________________________________________________________________
	
	def OnMixdown(self, widget):
		"""
		Called when the user clicks the Mixdown button.
		Calls the RunAction method on the MixdownActions
		in the action treeview (self.treeView). See RunAction in MixdownActions.py
		for more details.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""

		for row in self.treeViewModel:
			action = row[2]
			try:
				action.RunAction()
			except MixdownActions.MixdownActionException, e:
				text = _("An error occured while running the mixdown action: %s") % action.name
				text = "%s\n\n%s" % (text, e.message)
				dlg = gtk.MessageDialog(self.window,
				          gtk.DIALOG_DESTROY_WITH_PARENT,
				          gtk.MESSAGE_ERROR,
				          gtk.BUTTONS_CLOSE,
				          text)
				dlg.connect('response', lambda dlg, response: dlg.destroy())
				dlg.show()
				return
			
	
	#_____________________________________________________________________
	
	def CountRowsInTreeModel(self, treeModel):
		"""
		Called when the number of rows in the action treeview (self.treeView) is needed.

		Parameters:
			treeModel -- the model to use when counting the number of rows.

		Returns:
			rows -- number of rows in the action treeview.
		"""
		rows = treeModel.iter_n_children(None)
		return rows
	
	#_____________________________________________________________________


#=========================================================================

class AddMixdownActionDialog:
	"""
	This class allows the user to add a MixdownAction to the list of MixdownActions in
	the MixdownProfileDialog.
	"""
	
	#_____________________________________________________________________
	
	def __init__(self, profileDialog):
		"""
		Creates a new instance of MixdownProfileDialog.
		
		Parameters:
			profileDialog -- reference to the MixdownProfileDialog object which calls this class.
		"""
		self.profileDialog = profileDialog
		self.addActionDialogTree = gtk.glade.XML(Globals.GLADE_PATH, "AddMixdownActionDialog")
		
		signals = {
			"on_cancel_button_clicked" : self.OnCancelAction,
			"on_add_action_button_clicked" : self.OnAddAction,
		}
			
		self.addActionDialogTree.signal_autoconnect(signals)
	
		self.addActionDialog = self.addActionDialogTree.get_widget("AddMixdownActionDialog")
		self.treeView = self.addActionDialogTree.get_widget("treeview")
		self.actionLabel = self.addActionDialogTree.get_widget("action_label")
		self.addActionButton = self.addActionDialogTree.get_widget("add_action_button")
		
		self.treeModel = gtk.ListStore(gtk.gdk.Pixbuf, str, object) # pixbuf, details, class instance
		self.treeView.set_model(self.treeModel)
		
		self.treeView.append_column(gtk.TreeViewColumn(_("Icon"), gtk.CellRendererPixbuf(), pixbuf=0))
		self.treeView.append_column(gtk.TreeViewColumn(_("Name"), gtk.CellRendererText(), markup=1))
		
		self.treeViewSelection = self.treeView.get_selection()
		self.treeViewSelection.connect("changed", self.UpdateSelectedActionAppearence)
		self.treeViewSelection.set_mode(gtk.SELECTION_SINGLE)
		
		self.profileName = self.profileDialog.profileComboModel[self.profileDialog.profileCombo.get_active()][0]
		
		self.lastAction = None
		self.lastSelected = None
		
		# set some properties for the widgets
		self.addActionDialog.set_transient_for(self.profileDialog.window)
		self.addActionDialog.set_icon(self.profileDialog.window.get_icon())
		self.actionLabel.set_markup(_("Please select the mixdown actions you would like to add to mixdown profile <b>%s.</b>") % self.profileName)
	
		self.PopulateActionModel()
	
	#_____________________________________________________________________
	
	def UpdateSelectedActionAppearence(self, widget):
		"""
		Called when a MixdownAction is selected in the add action treeview (self.treeView).
		Changes the appearence of the selected action.
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		selected = self.treeViewSelection.get_selected()
		action = self.treeModel [selected[1]] [2]
		
		# set the foreground colour of the text to white
		newText = "<span size='larger' weight='bold'>%s</span>\n" % action.name  \
		+ "<span size='smaller' foreground='white'>%s</span>" % action.description
		self.treeModel [selected[1]] [1] = newText
		
		# modify the last selected item in the model to use the default colour grey
		if self.lastSelected:
			oldText = "<span size='larger' weight='bold'>%s</span>\n" % self.lastAction.name  \
			+ "<span size='smaller' foreground='dim grey'>%s</span>" % self.lastAction.description
			self.treeModel [self.lastSelected[1]] [1] = oldText
		
		self.lastAction = action
		self.lastSelected = selected
	
	#_____________________________________________________________________

	def ReturnAllActions(self):
		"""
		Returns all actions in MixdownActions.py, excluding the MixdownAction class.
		
		Returns:
			actionList -- list of MixdownAction instances
		"""
		actionList = []
		for action in self.profileDialog.mainapp.registerMixdownActionAPI.ReturnAllActions():
			# we have to pass Project to ExportAsFileType for it to work
			if action.__name__ == "ExportAsFileType":
				actionList.append( action(self.profileDialog.project) )
			else:
				actionList.append( action() )
		# a list of MixdownAction instances should be returned
		return actionList

	#_____________________________________________________________________

	def PopulateActionModel(self):
		"""
		Called when the action model (self.treeModel) needs to be populated.
		"""
		for action in self.ReturnAllActions():
			self.treeModel.append( self.profileDialog.ReturnActionDisplayDetails(action) )
		
	#_____________________________________________________________________

	def OnAddAction(self, widget):
		"""
		Called when the Add Action button is clicked.
		Adds the selected MixdownAction to the profile dialog's action
		model (self.profileDialog.treeViewModel).
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		if self.treeViewSelection.count_selected_rows() > 0:
			selected = self.treeViewSelection.get_selected()
			action = self.treeModel [selected[1]] [2]
			self.profileDialog.AddActionToActionModel(self.profileName, action)
		else:
			return
		self.addActionDialog.destroy()
			
	#_____________________________________________________________________
	
	def OnCancelAction(self, widget):
		"""
		Called when the Cancel button is clicked.
		Destroys the Add Mixdown Action dialog
		
		Parameters:
			widget -- reserved for GTK callbacks, don't use it explicitly.
		"""
		self.addActionDialog.destroy()
	
	#_____________________________________________________________________

#=========================================================================
